<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_checkout_field_meta_box() {
  add_meta_box('quickcab_checkout_field_options', esc_html__('Checkout Field Options', 'quickcab'),  'quickcab_render_checkout_field_options_meta_box', array('qc_checkout_field'), 'normal', 'default');
}
add_action('add_meta_boxes', 'quickcab_add_checkout_field_meta_box');

function quickcab_render_checkout_field_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_checkout_field_options_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_checkout_field_meta_box_nonce');
}

function quickcab_render_checkout_field_options_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !$is_new ) {
    $checkout_field = new QuickCab_Checkout_Field($post_id);
  }

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_checkout_field_metabox_string_row(
        'quickcab_checkout_field_label',
        $post_id,
        array(
          'text' => esc_html__('Input Label', 'quickcab')
        ),
        array(
          'required' => true
        ),
        array(
          'text' => esc_html__('The label that will display next to the input', 'quickcab')
        )
      );
      quickcab_render_checkout_field_metabox_string_row(
        'quickcab_checkout_field_placeholder',
        $post_id,
        array(
          'text' => esc_html__('Placeholder', 'quickcab')
        ),
        array(
          'required' => true
        ),
        array(
          'text' => esc_html__('The placeholder value for the input', 'quickcab')
        )
      );
    ?></tbody>
    <tbody>
      <tr>
        <th scope="row">
          <label for="quickcab_checkout_field_is_required"><?php
            echo esc_html__('Required?', 'quickcab');
          ?></label>
        </th>
        <td>
          <input name="quickcab_checkout_field_is_required" type="checkbox" id="quickcab_checkout_field_is_required" <?php if (!$is_new && $checkout_field->is_required()) { ?>checked<?php } ?>>

          <p class="description"><?php
            echo esc_html__('Is the customer required to enter an input to this field?', 'quickcab');
          ?></p>
        </td>
      </tr>
      <tr>
        <th scope="row">
          <label for="quickcab_checkout_field_show_in_email"><?php
            echo esc_html__('Show in order confirmation email?', 'quickcab');
          ?></label>
        </th>
        <td>
          <input name="quickcab_checkout_field_show_in_email" type="checkbox" id="quickcab_checkout_field_show_in_email" <?php if (!$is_new && $checkout_field->show_in_email()) { ?>checked<?php } ?>>

          <p class="description"><?php
            echo esc_html__('Do you want this custom field to display in the order confirmation email that customers get?', 'quickcab');
          ?></p>
        </td>
      </tr>
      <tr>
        <th scope="row">
          <label for="quickcab_checkout_field_input_type"><?php
            echo esc_html__('Input Type', 'quickcab');
          ?></label>
        </th>

        <td>
          <select name="quickcab_checkout_field_input_type" id="quickcab_checkout_field_input_type">
            <option value="text" <?php if (!$is_new && $checkout_field->get_input_type() == 'text') { ?>selected<?php } ?>><?php
              echo esc_html__('Text', 'quickcab');
            ?></option>
            <option value="textarea" <?php if (!$is_new && $checkout_field->get_input_type() == 'textarea') { ?>selected<?php } ?>><?php
              echo esc_html__('Textarea', 'quickcab') . ' (' . esc_html__('large text box', 'quickcab') . ')';
            ?></option>
            <option value="select" <?php if (!$is_new && $checkout_field->get_input_type() == 'select') { ?>selected<?php } ?>><?php
              echo esc_html__('Select', 'quickcab');
            ?></option>
          </select>

          <p class="description"><?php
            echo esc_html__('How should the customer input data for this field?', 'quickcab');
          ?></p>
        </td>
      </tr>
    </tbody>
    <tbody <?php if ($is_new || ($checkout_field->get_input_type() == 'text' || $checkout_field->get_input_type() == 'textarea')) { ?>style="display:none;"<?php } ?> id="quickcab_checkout_field_input_options_tbody">
      <tr>
        <th scope="row">
          <label for="quickcab_checkout_field_is_required"><?php
            echo esc_html__('Options', 'quickcab');
          ?></label>
        </th>

        <td>
          <ul class="quickcab-checkout-field-options-list"><?php
            if ( $is_new || empty($checkout_field->get_input_options()) ) {
              ?><li class="quickcab-checkout-field-option-group">
                <input name="quickcab_checkout_field_input_options[]" type="text" id="quickcab_checkout_field_input_options" class="regular-text">
                <a href="#" class="button quickcab-checkout-field-remove-option"><?php
                  echo esc_html__('remove', 'quickcab');
                ?></a>
              </li><?php
            } else {
              foreach ( $checkout_field->get_input_options() as $input_option ) {
                ?><li class="quickcab-checkout-field-option-group">
                  <input name="quickcab_checkout_field_input_options[]" type="text" id="quickcab_checkout_field_input_options" class="regular-text" value="<?php echo $input_option; ?>">
                  <a href="#" class="button quickcab-checkout-field-remove-option"><?php
                    echo esc_html__('remove', 'quickcab');
                  ?></a>
                </li><?php
              }
            }
          ?></ul>

          <a href="#" class="button quickcab-checkout-field-add-option"><?php
            echo esc_html__('Add option', 'quickcab');
          ?></a>

          <p class="description"><?php
            echo esc_html__('The options the user can choose from for this input', 'quickcab');
          ?></p>
        </td>
      </tr>
    </tbody>
  </table><?php
}
add_action('quickcab_meta_box_checkout_field_options_fields', 'quickcab_render_checkout_field_options_fields');

function quickcab_checkout_field_metabox_fields() {
  return apply_filters('quickcab_metabox_checkout_field_fields_save', array(
    'quickcab_checkout_field_label',
    'quickcab_checkout_field_placeholder',
    'quickcab_checkout_field_is_required',
    'quickcab_checkout_field_show_in_email',
    'quickcab_checkout_field_input_type',
    'quickcab_checkout_field_input_options'
  ));
}

function quickcab_checkout_field_meta_box_save($post_id, $post) {
  if ( !isset($_POST['quickcab_checkout_field_meta_box_nonce']) || !wp_verify_nonce($_POST['quickcab_checkout_field_meta_box_nonce'], basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $checkout_field = new QuickCab_Checkout_Field($post_id);

  // The default fields that get saved
  foreach ( quickcab_checkout_field_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_checkout_field_', '', $field);

    if ( empty($_POST[$field]) ) {
      $checkout_field->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_checkout_field_metabox_save_' . $meta_name, $_POST[$field], $post_id);
      $checkout_field->update_meta($meta_name, $new);
    }
  }

  $key = uniqid('quickcab_');

  $results_with_key = get_posts(array(
    'posts_per_page' => 1, // we only want to check if any exists, so don't need to get all of them
    'meta_key' => '_quickcab_checkout_field_unique_key',
    'meta_value' => $key,
    'fields' => 'ids', // we don't need it's content, etc.
  ));

  while (count($results_with_key) > 0) {
    $key = uniqid('quickcab_');

    $results_with_key = get_posts(array(
      'posts_per_page' => 1, // we only want to check if any exists, so don't need to get all of them
      'meta_key' => '_quickcab_checkout_field_unique_key',
      'meta_value' => $key,
      'fields' => 'ids', // we don't need it's content, etc.
    ));
  }

  $checkout_field->update_meta('unique_key', $key);

  do_action('quickcab_save_checkout_field', $post_id, $post);
}
add_action('save_post', 'quickcab_checkout_field_meta_box_save', 10, 2);

function quickcab_serialize_input_options($input_options) {
  return serialize($input_options);
}
add_filter('quickcab_checkout_field_metabox_save_input_options', 'quickcab_serialize_input_options');

/**
 * Update the post title to the label of the field
 *
 * @since  1.0
 * @param  int     $post_id  ID of the post
 * @param  WP_Post $post     the WP_Post
 * @return void
 */
function quickcab_checkout_field_update_title($post_id, $post) {
  global $wpdb;
  $checkout_field = new QuickCab_Checkout_Field($post_id);
  $title = $checkout_field->get_label();

  $wpdb->update(
    $wpdb->posts,
    array('post_title' => $title),
    array('ID' => $post_id)
  );
}
add_action('quickcab_save_checkout_field', 'quickcab_checkout_field_update_title', 10, 2);
