<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_geofence_add_geofence_meta_box() {
  add_meta_box('quickcab_geofence_shapes', esc_html__('Geofence Shapes', 'quickcab'),  'quickcab_geofence_render_shapes_meta_box', array('geofence_rule'), 'side', 'low');
  add_meta_box('quickcab_geofence_maps', esc_html__('Locations', 'quickcab'),  'quickcab_geofence_render_maps_meta_box', array('geofence_rule'), 'normal', 'high');
  add_meta_box('quickcab_geofence_options', esc_html__('Options', 'quickcab'),  'quickcab_geofence_render_options_meta_box', array('geofence_rule'), 'normal', 'high');
}
add_action('add_meta_boxes', 'quickcab_geofence_add_geofence_meta_box');

function quickcab_geofence_render_shapes_meta_box() {
  global $post;

  do_action('quickcab_geofence_meta_box_shapes', $post->ID);
}

function quickcab_geofence_render_maps_meta_box() {
  global $post;

  do_action('quickcab_geofence_meta_box_maps', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_geofence_meta_box_nonce');
}

function quickcab_geofence_render_options_meta_box() {
  global $post;

  do_action('quickcab_geofence_meta_box_options', $post->ID);
}

function quickcab_geofence_render_maps($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $geofence_rule = new QuickCab_Geofence_Rule($post_id);
  $geofence_shapes = QuickCab_Geofence_Shape::find_all_available();

  ?><input type="hidden" name="quickcab_geofence_starting_coords" id="quickcab_geofence_starting_coords" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_starting_coords_string()); ?>"<?php } ?>>
  <input type="hidden" name="quickcab_geofence_starting_polygon_center" id="quickcab_geofence_starting_polygon_center" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_starting_polygon_center()); ?>"<?php } ?>>
  <input type="hidden" name="quickcab_geofence_starting_polygon_zoom" id="quickcab_geofence_starting_polygon_zoom" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_starting_polygon_zoom()); ?>"<?php } ?>>
  <input type="hidden" name="quickcab_geofence_destination_coords" id="quickcab_geofence_destination_coords" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_destination_coords_string()); ?>"<?php } ?>>
  <input type="hidden" name="quickcab_geofence_destination_polygon_center" id="quickcab_geofence_destination_polygon_center" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_destination_polygon_center()); ?>"<?php } ?>>
  <input type="hidden" name="quickcab_geofence_destination_polygon_zoom" id="quickcab_geofence_destination_polygon_zoom" <?php if (!$is_new) { ?>value="<?php echo esc_attr($geofence_rule->get_destination_polygon_zoom()); ?>"<?php } ?>>

  <div class="quickcab-row">
    <div class="quickcab-notice" style="display: none;"></div>
    <div class="qc-left-column">
      <h2><?php echo esc_html__('Location 1', 'quickcab'); ?></h2>
      <div class="quickcab-pricing-condition-map-actions">
        <a href="#" id="quickcab_geofence_starting_delete_shape" class="button button-default button-large" <?php if ($geofence_rule->starting_area_anywhere()) { ?>disabled<?php } ?>><?php
          echo esc_html__('Remove Shape From Map', 'quickcab');
        ?></a>
        <a href="#" id="quickcab_geofence_starting_save_shape" class="button button-default button-large" <?php if (empty($geofence_rule->get_meta('starting_coords'))) { ?>disabled<?php } ?>><?php
          echo esc_html__('Save Shape', 'quickcab');
        ?></a>
        <a href="#" id="quickcab_geofence_starting_import_shape" class="button button-default button-large"><?php
          echo esc_html__('Import Shape', 'quickcab');
        ?></a>
      </div>
      <div id="quickcab_geofence_starting_save_shape_actions" style="display: none;">
        <input type="text" name="quickcab_geofence_starting_shape_name" id="quickcab_geofence_starting_shape_name" placeholder="<?php echo esc_html__('Shape Name', 'quickcab'); ?>">
        <a href="#" class="button button-primary" id="quickcab_geofence_starting_save_shape_button"><?php
          echo esc_html__('Save', 'quickcab');
        ?></a>
      </div>
      <div id="quickcab_geofence_starting_import_shape_actions" style="display: none;">
        <select id="quickcab_geofence_starting_shape">
          <option disabled selected><?php
            echo esc_html__('Select a shape', 'quickcab') . '...';
          ?></option><?php

          foreach ( $geofence_shapes as $geofence_shape ) {
            ?><option value="<?php echo esc_attr($geofence_shape->get_id()); ?>" data-coordinates="<?php echo esc_attr($geofence_shape->get_serialised_coordinates()); ?>"><?php
              echo esc_html($geofence_shape->get_name());
            ?></option><?php
          }
        ?></select>
        <a href="#" class="button button-primary" id="quickcab_geofence_starting_import_shape_button"><?php
          echo esc_html__('Import', 'quickcab');
        ?></a>
      </div>
      <div class="quickcab-geofence-location-selector">
        <label for="quickcab_geofence_starting_map_anywhere_map">
          <input type="radio" name="quickcab_geofence_starting_map_anywhere" value="map" id="quickcab_geofence_starting_map_anywhere_map" <?php if ($is_new || $geofence_rule->starting_area_map()) { ?>checked<?php } ?>>
          <?php echo esc_html__('Specific Location', 'quickcab'); ?>
        </label>
        <label for="quickcab_geofence_starting_map_anywhere_anywhere">
          <input type="radio" name="quickcab_geofence_starting_map_anywhere" value="anywhere" id="quickcab_geofence_starting_map_anywhere_anywhere" <?php if (!$is_new && $geofence_rule->starting_area_anywhere()) { ?>checked<?php } ?>>
          <?php echo esc_html__('Anywhere', 'quickcab'); ?>
        </label>
      </div>
      <div class="quickcab-geofence-map-container">
        <div class="quickcab-geofence-map-overlay -starting" <?php if ($is_new || $geofence_rule->starting_area_map()) { ?>style="display: none;"<?php } ?>><?php
          echo esc_html__('Anywhere', 'quickcab');
        ?></div>
        <div id="quickcab_geofence_starting_map" class="quickcab-geofence-map"></div>
      </div>
    </div>
    <div class="qc-right-column">
      <h2><?php echo esc_html__('Location 2', 'quickcab'); ?></h2>
      <div class="quickcab-pricing-condition-map-actions">
        <a href="#" id="quickcab_geofence_destination_delete_shape" class="button button-default button-large" <?php if ($geofence_rule->destination_area_anywhere()) { ?>disabled<?php } ?>><?php
          echo esc_html__('Remove Shape From Map', 'quickcab');
        ?></a>
        <a href="#" id="quickcab_geofence_destination_save_shape" class="button button-default button-large" <?php if (empty($geofence_rule->get_meta('destination_coords'))) { ?>disabled<?php } ?>><?php
          echo esc_html__('Save Shape', 'quickcab');
        ?></a>
        <a href="#" id="quickcab_geofence_destination_import_shape" class="button button-default button-large"><?php
          echo esc_html__('Import Shape', 'quickcab');
        ?></a>
      </div>
      <div id="quickcab_geofence_destination_save_shape_actions" style="display: none;">
        <input type="text" name="quickcab_geofence_destination_shape_name" id="quickcab_geofence_destination_shape_name" placeholder="<?php echo esc_html__('Shape Name', 'quickcab'); ?>">
        <a href="#" class="button button-primary" id="quickcab_geofence_destination_save_shape_button"><?php
          echo esc_html__('Save', 'quickcab');
        ?></a>
      </div>
      <div id="quickcab_geofence_destination_import_shape_actions" style="display: none;">
        <select id="quickcab_geofence_destination_shape">
          <option disabled selected><?php
            echo esc_html__('Select a shape', 'quickcab') . '...';
          ?></option><?php

          foreach ( $geofence_shapes as $geofence_shape ) {
            ?><option value="<?php echo esc_attr($geofence_shape->get_id()); ?>" data-coordinates="<?php echo esc_attr($geofence_shape->get_serialised_coordinates()); ?>"><?php
              echo esc_html($geofence_shape->get_name());
            ?></option><?php
          }
        ?></select>
        <a href="#" class="button button-primary" id="quickcab_geofence_destination_import_shape_button"><?php
          echo esc_html__('Import', 'quickcab');
        ?></a>
      </div>
      <div class="quickcab-geofence-location-selector">
        <label for="quickcab_geofence_destination_map_anywhere_map">
          <input type="radio" name="quickcab_geofence_destination_map_anywhere" value="map" id="quickcab_geofence_destination_map_anywhere_map" <?php if ($is_new || $geofence_rule->destination_area_map()) { ?>checked<?php } ?>>
          <?php echo esc_html__('Specific Location', 'quickcab'); ?>
        </label>
        <label for="quickcab_geofence_destination_map_anywhere_anywhere">
          <input type="radio" name="quickcab_geofence_destination_map_anywhere" value="anywhere" id="quickcab_geofence_destination_map_anywhere_anywhere" <?php if (!$is_new && $geofence_rule->destination_area_anywhere()) { ?>checked<?php } ?>>
          <?php echo esc_html__('Anywhere', 'quickcab'); ?>
        </label>
      </div>
      <div class="quickcab-geofence-map-container">
        <div class="quickcab-geofence-map-overlay -destination" <?php if ($is_new || $geofence_rule->destination_area_map()) { ?>style="display: none;"<?php } ?>><?php
          echo esc_html__('Anywhere', 'quickcab');
        ?></div>
        <div id="quickcab_geofence_destination_map" class="quickcab-geofence-map"></div>
      </div>
    </div>
  </div><?php
}
add_action('quickcab_geofence_meta_box_maps', 'quickcab_geofence_render_maps', 10);

function quickcab_geofence_render_options($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  $vehicles = QuickCab_Vehicle::find_all();
  $geofence_rule = new QuickCab_Geofence_Rule($post_id);
  $p_struct = $geofence_rule->get_meta('pricing_structure');

  ?><table class="form-table">
    <tbody>
      <tr>
        <th scope="row">
          <label for="quickcab_geofence_pricing_structure"><?php echo esc_html__('Increase Price/Fixed Price', 'quickcab'); ?></label>
        </th>
        <td>
          <select name="quickcab_geofence_pricing_structure" id="quickcab_geofence_pricing_structure">
            <option value="fixed-price" <?php if ($p_struct == 'fixed-price') { ?>selected<?php } ?>><?php
              echo esc_html__('Fixed Price', 'quickcab');
            ?></option>
            <option value="increased-price" <?php if ($p_struct == 'increased-price') { ?>selected<?php } ?>><?php
              echo esc_html__('Increased Price', 'quickcab');
            ?></option>
          </select>
        </td>
      </tr>
    </tbody>
    <tbody id="quickcab_geofence_ignore_surge_pricing_rule_cont" <?php if (!$is_new && $geofence_rule->is_increased_price()) { ?>style="display: none;"<?php } ?>><?php
      quickcab_render_geofence_metabox_checkbox_row(
        'quickcab_geofence_ignore_surge_pricing_rule',
        $post_id,
        array(
          'text' => esc_html__('Ignore surge price rules', 'quickcab')
        ),
        array(
          'size' => 'medium'
        ),
        array(
          'text' => esc_html__('If yes, any surge price increases will be ignored. If no and a surge pricing rule applies to the journey, the journey will be charged the fixed price specified + the surge price increase.', 'quickcab')
        )
      );
    ?></tbody>
    <tbody id="quickcab_geofence_fixed_price" <?php if ($geofence_rule->is_increased_price()) { ?>style="display: none;"<?php } ?>><?php
      quickcab_render_geofence_metabox_price_row(
        'quickcab_geofence_fixed_price_amount',
        $post_id,
        array(
          'text' => esc_html__('Fixed Price', 'quickcab')
        ),
        array(
          'size' => 'medium'
        )
      );
    ?></tbody>
    <tbody id="quickcab_geofence_increased_price" <?php if ($is_new || !$geofence_rule->is_increased_price()) { ?>style="display: none;"<?php } ?>><?php
      quickcab_render_geofence_metabox_price_row(
        'quickcab_geofence_increase_price_amount',
        $post_id,
        array(
          'text' => esc_html__('Increase Price By', 'quickcab')
        ),
        array(
          'size'   => 'medium',
          'prefix' => '+'
        )
      );
    ?></tbody>
    <tbody>
      <tr>
        <th scope="row">
          <label for="quickcab_geofence_vehicle_ids"><?php echo esc_html__('Vehicle', 'quickcab'); ?></label>
        </th>
        <td>
          <fieldset>
            <legend class="screen-reader-text">
              <span><?php echo esc_html__('All Vehicles', 'quickcab'); ?></span>
            </legend>
            <label for="quickcab_geofence_vehicle_0">
              <input name="quickcab_geofence_vehicle_ids[]" type="checkbox" id="quickcab_geofence_vehicle_0" value="0" <?php if (!$is_new && in_array(0, $geofence_rule->get_vehicle_ids())) { ?>checked<?php } ?>>
              <?php echo esc_html__('All Vehicles', 'quickcab'); ?>
            </label>
          </fieldset><?php

          foreach ( $vehicles as $vehicle ) {
            ?><fieldset>
              <legend class="screen-reader-text">
                <span><?php echo esc_html($vehicle->get_name()); ?></span>
              </legend>
              <label for="quickcab_geofence_vehicle_<?php echo esc_html($vehicle->get_ID()); ?>">
                <input name="quickcab_geofence_vehicle_ids[]" type="checkbox" id="quickcab_geofence_vehicle_<?php echo esc_attr($vehicle->get_ID()); ?>" value="<?php echo esc_attr($vehicle->get_ID()); ?>" <?php if (!$is_new && (in_array($vehicle->get_ID(), $geofence_rule->get_vehicle_ids()) || in_array(0, $geofence_rule->get_vehicle_ids()))) { ?>checked<?php } ?> class="quickcab__geofence--vehicles-checkboxes">
                <?php echo esc_html($vehicle->get_name()); ?>
              </label>
            </fieldset><?php
          }
          ?><p class="description"><?php
            echo esc_html__('Which vehicles do you want this pricing condition to apply to?', 'quickcab');
          ?></p>
        </td>
      </tr>
      <tr>
        <th scope="row">
          <label for="quickcab_geofence_direction"><?php
            echo esc_html__('Direction', 'quickcab');
          ?></label>
        </th>
        <td>
          <select name="quickcab_geofence_direction" id="quickcab_geofence_direction">
            <option value="one-way" <?php if (!$is_new && $geofence_rule->get_direction() == 'one-way') { ?>selected<?php } ?>><?php
              echo esc_html__('Origin => Destination (One Direction)', 'quickcab');
            ?></option>
            <option value="both-ways" <?php if (!$is_new && $geofence_rule->get_direction() == 'both-ways') { ?>selected<?php } ?>><?php
              echo esc_html__('Origin <=> Destination (Both Directions)', 'quickcab');
            ?></option>
          </select>
          <p class="description"><?php
            echo esc_html__('Does this rule apply for Location A => Location B only, or will it apply to Location B => Location A as well?', 'quickcab');
          ?></p>
        </td>
      </tr>
    </tbody>
  </table><?php
}
add_action('quickcab_geofence_meta_box_options', 'quickcab_geofence_render_options', 10);

function quickcab_geofence_render_shapes($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  $geofence_rule = new QuickCab_Geofence_Rule($post_id);
  $shapes = QuickCab_Geofence_Shape::find_all_available();

  ?><ul class="quickcab-geofence-metabox-shape-list"><?php
    foreach ( $shapes as $shape ) {
      ?><li><?php
        echo esc_html($shape->get_name());
        ?> <a href="#" class="quickcab-delete-geofence-shape text-red" data-geofence-shape-id="<?php echo esc_attr($shape->get_id()); ?>"><?php
          echo '(' . esc_html__('delete', 'quickcab') . ')';
        ?></a>
      </li><?php
    }
  ?></ul><?php
}
add_action('quickcab_geofence_meta_box_shapes', 'quickcab_geofence_render_shapes');

/**
 * The fields in the metaboxes that should be saved
 *
 * @since  1.0
 * @return array  array of field names to be saved
 */
function quickcab_geofence_metabox_fields() {
  return apply_filters('quickcab_geofence_metabox_fields_save', array(
    'quickcab_geofence_starting_coords',
    'quickcab_geofence_destination_coords',
    'quickcab_geofence_pricing_structure',
    'quickcab_geofence_ignore_surge_pricing_rule',
    'quickcab_geofence_fixed_price_amount',
    'quickcab_geofence_increase_price_amount',
    'quickcab_geofence_vehicle_ids',
    'quickcab_geofence_direction',
    'quickcab_geofence_starting_polygon_center',
    'quickcab_geofence_starting_polygon_zoom',
    'quickcab_geofence_destination_polygon_center',
    'quickcab_geofence_destination_polygon_zoom',
    'quickcab_geofence_starting_map_anywhere',
    'quickcab_geofence_destination_map_anywhere'
  ));
}

/**
 * Run after the post is saved, saves all data in metaboxes
 *
 * @since  1.0
 * @param  int     $post_id  ID of the post
 * @param  WP_POST $post     the post being saved
 * @return void
 */
function quickcab_geofence_meta_box_save($post_id, $post) {
  if ( !isset($_POST['quickcab_geofence_meta_box_nonce']) || !wp_verify_nonce($_POST['quickcab_geofence_meta_box_nonce'], basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $geofence_rule = new QuickCab_Geofence_Rule($post_id);

  foreach ( quickcab_geofence_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_geofence_', '', $field);

    if ( empty(wp_unslash($_POST[$field])) ) {
      $geofence_rule->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_geofence_metabox_save_' . $meta_name, $_POST[$field], $post_id);
      $geofence_rule->update_meta($meta_name, $new);
    }
  }

  if ( quickcab_get_option('enable_static_map_geofence_preview') == 'on' ) {
    quickcab_calculate_geofence_image($post);
  }

  do_action('quickcab_save_geofence_rule', $post_id, $post);
}
add_action('save_post', 'quickcab_geofence_meta_box_save', 10, 2);

/**
 * Created the names etc. for the images of the geofence areas to be saved
 *
 * @since  1.0
 * @param  WP_Post $post  the WP_Post
 * @return void
 */
function quickcab_calculate_geofence_image($post) {
  require_once ABSPATH . 'wp-admin/includes/image.php';

  $geofence = new QuickCab_Geofence_Rule($post->ID);

  $starting_filepath = $geofence->get_starting_polygon_maps_image_url();
  $starting_options = array(
    'title' => 'starting-geofence-map',
    'filename' => 'starting_geofence_' . date('Ymd_His') . '.jpg'
  );

  $ending_filepath = $geofence->get_destination_polygon_maps_image_url();
  $ending_options = array(
    'title' => 'destination-geofence-map',
    'filename' => 'destination_geofence_' . date('Ymd_His') . '.jpg'
  );

  quickcab_save_geofence_image($starting_filepath, $starting_options, $post->ID);
  quickcab_save_geofence_image($ending_filepath, $ending_options, $post->ID);
}

/**
 * Save the static image of the polygon to this server. This is done so we do
 * not need to do 2 reqests per rule on the index page
 *
 * @since  1.0
 * @param  string $image_url      the url of the image
 * @param  array  $image_options  options for the image
 * @param  int    $post_id        the id of the post to attach the image to
 * @return int                    the attachment id
 */
function quickcab_save_geofence_image($image_url, $image_options, $post_id) {
  require_once ABSPATH . 'wp-admin/includes/image.php';

  $children_images = get_attached_media('image', $post_id);

  foreach ( $children_images as $image ) {
    if ( $image->post_title == $image_options['title'] ) {
      wp_delete_post($image->ID);
    }
  }

  $get_image = wp_remote_get($image_url);

  if ( $get_image['response']['code'] == 403 ) {
    return false;
  }

  $image_type = wp_remote_retrieve_header($get_image, 'Content-Type');

  if ( !$image_type ) {
    return false;
  }

  $mirror = wp_upload_bits($image_options['filename'], '', wp_remote_retrieve_body($get_image));
  $attachment = array(
    'post_title'     => $image_options['title'],
    'post_mime_type' => $image_type
  );

  $attach_id = wp_insert_attachment($attachment, $mirror['file'], $post_id);
  $attach_data = wp_generate_attachment_metadata($attach_id, $mirror['file']);

  wp_update_attachment_metadata($attach_id, $attach_data);

  return $attach_id;
}
