<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_register_settings() {

  if ( get_option('quickcab_settings') == false ) {
    add_option('quickcab_settings');
  }

  foreach ( quickcab_get_registered_settings() as $tab_id => $tab ) {
    foreach ( $tab['sections'] as $section_id => $section ) {

      add_settings_section(
        'quickcab_settings_' . $tab_id . '_' . $section_id,
        null,
        null,
        'quickcab_settings_' . $tab_id . '_' . $section_id
      );

      foreach ( $section['settings'] as $setting ) {

        $args = wp_parse_args($setting, array(
          'section'     => $section_id,
          'id'          => null,
          'name'        => '',
          'size'        => null,
          'options'     => '',
          'min'         => '',
          'max'         => '',
          'step'        => '',
          'chosen'      => null,
          'placeholder' => null,
          'class'       => '',
          'helper_id'   => '',
          'helper_text' => '',
          'prefix'      => '',
          'suffix'      => '',
          'is_price'    => null,
          'disabled'    => null,
          'cols'        => 50,
          'rows'        => 5,
          'dependency'  => array()
        ));

        add_settings_field(
          'quickcab_settings[' . $args['id'] . ']',
          $args['name'],
          function_exists('quickcab_' . $args['type'] . '_callback') ? 'quickcab_' . $args['type'] . '_callback' : 'quickcab_no_callback',
          'quickcab_settings_' . $tab_id . '_' . $section_id,
          'quickcab_settings_' . $tab_id . '_' . $section_id,
          $args
        );
      }
    }
  }

  // Creates our settings in the options table
  register_setting('quickcab_settings', 'quickcab_settings', 'quickcab_settings_sanitize');
}
add_action('admin_init', 'quickcab_register_settings');

function quickcab_get_registered_settings() {
  $units = quickcab_get_option('distance_units');
  $external_ip = file_get_contents("http://ipecho.net/plain");

  $quickcab_settings = array(
    'general' => apply_filters('quickcab_settings_general', array(
      'title' => esc_html__('General', 'quickcab'),
      'sections' => array(
        'main' => array(
          'title' => esc_html__('General Settings', 'quikcab'),
          'settings' => array(
            'google_maps_api_key_info' => array(
              'type' => 'descriptive_text',
              'id' => 'google_maps_api_key_info',
              'name' => '',
              'desc' => sprintf(
                esc_html__('You must get an API key for Google Maps. To create one, %s. If you are having issues with Google Maps, common problems and their solutions can be found %s. If the issue is still not fixed %s to open the JavaScript console on the page containing your QuickCab booking form to view the error.', 'quickcab'),
                '<a href="https://developers.google.com/maps/documentation/javascript/get-api-key" target="_blank">' . esc_html__('click here', 'quickcab') . '</a>',
                '<a href="https://developers.google.com/maps/documentation/javascript/error-messages?utm_source=maps_js&utm_medium=degraded&utm_campaign=billing#api-key-and-billing-errors" target="_blank">' . esc_html__('here', 'quickcab') . '</a>',
                '<a href="https://webmasters.stackexchange.com/questions/8525/how-do-i-open-the-javascript-console-in-different-browsers" target="_blank">' . esc_html__('follow this guide', 'quickcab') . '</a>'
              )
            ),
            'google_maps_api_key' => array(
              'id'          => 'google_maps_api_key',
              'name'        => esc_html__('API Key 1', 'quickcab'),
              'type'        => 'text',
              'placeholder' => esc_html__('API Key 1', 'quickcab'),
              'helper_text' => __('If you want to restrict this API key, you must restrict it to <strong>HTTP referrers</strong> since this key is used for the front-end libraries.', 'quickcab')
            ),
            'google_static_maps_api_key' => array(
              'id'          => 'google_static_maps_api_key',
              'name'        => esc_html__('API Key 2', 'quickcab'),
              'type'        => 'text',
              'placeholder' => esc_html__('API Key 2', 'quickcab'),
              'helper_text' => sprintf(
                __('If you want to restrict this API key, you must restrict it to <strong>IP addresses</strong> (the IP address for this server is %s). This is the API key for handling all server-side interactions with Google Maps that QuickCab does. It must be different to the API key you entered above.', 'quickcab'),
                $external_ip
              )
            ),
            'enable_static_map_geofence_preview' => array(
              'id'          => 'enable_static_map_geofence_preview',
              'name'        => esc_html__('Enable Geofence Static Map Preview', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__("It is not required but if this is enabled, you will be able to see an image of your Geofence shapes on the 'Geofence Rules' page.", 'quickcab')
            ),
            'distance_units' => array(
              'id'           => 'distance_units',
              'name'         => esc_html__('Units', 'quickcab'),
              'type'         => 'select',
              'helper_text'  => esc_html__('The units used for distance calculation.', 'quickcab'),
              'options'      => array(
                'miles'      => esc_html__('Miles', 'quickcab'),
                'kilometers' => esc_html__('Kilometers', 'quickcab')
              )
            ),
            'map_starting_location' => array(
              'id'           => 'map_starting_location',
              'name'         => esc_html__('Map starting location', 'quickcab'),
              'type'         => 'google_autocomplete',
              'helper_text'  => esc_html__('An area you want to map to initialise around (for example on the form). This can be a town, city, state, country or any other address.', 'quickcab')
            ),
            'default_calendar_period' => array(
              'id'           => 'default_calendar_period',
              'name'         => esc_html__('Default calendar period', 'quickcab'),
              'type'         => 'select',
              'helper_text'  => esc_html__('The default time period the calendar will open on.', 'quickcab'),
              'options'      => array(
                'dayGridMonth' => esc_html__('Month', 'quickcab'),
                'dayGridWeek'  => esc_html__('Week', 'quickcab'),
                'timeGridDay'  => esc_html__('Day', 'quickcab'),
                'listWeek'     => esc_html__('List', 'quickcab')
              )
            ),
            'currency' => array(
              'id' => 'currency',
              'name' => esc_html__('Currency', 'quickcab'),
              'type' => 'descriptive_text',
              'desc' => __('Payments are handled by WooCommerce, please change currency options in the WooCommerce settings.', 'quickcab')
            )
          )
        ),
      )
    )),
    'journeys' => apply_filters('quickcab_settings_journeys', array(
      'title' => esc_html__('Journeys', 'quickcab'),
      'sections' => array(
        'main' => array(
          'title' => esc_html__('Fare Settings', 'quickcab'),
          'settings' => array(
            'journey_options' => array(
              'id'   => 'journey_options',
              'name' => '<h3>' . esc_html__('Journey Options', 'quickcab') . '</h3>',
              'type' => 'header'
            ),
            'use_base_location_for_availability' => array(
              'id'          => 'use_base_location_for_availability',
              'name'        => esc_html__('Use Base Location for Vehicle Availability', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__("If this is enabled the availability of a given vehicle will take into account the time taken for vehicles to get from your base location to the journey collection address and then to return there from the destination address when the journey has ended. NOTE: This may slow down quote generation in the booking form, depending on your servers performance.", 'quickcab')
            ),
            'vehicle_base_location' => array(
              'id'           => 'vehicle_base_location',
              'name'         => esc_html__('Vehicle Base Location', 'quickcab'),
              'type'         => 'google_autocomplete',
              'helper_text'  => esc_html__('This is the location your vehicles will start from when they drive to a collection point. This is used to ensure a vehicle is not double-booked while travelling to or from another reservation.', 'quickcab')
            ),
            'return_discount_rate' => array(
              'id'          => 'return_discount_rate',
              'name'        => esc_html__('Return Discount Rate', 'quickcab'),
              'type'        => 'number',
              'min'         => 0,
              'max'         => 100,
              'step'        => 0.1,
              'size'        => 'small',
              'helper_text' => esc_html__('The discount applied to the return leg of the journey.', 'quickcab'),
              'suffix'      => '%'
            ),
            'min_max_fare_header' => array(
              'id'   => 'min_max_fare_header',
              'name' => '<h3>' . esc_html__('Min/Max Fare', 'quickcab') . '</h3>',
              'type' => 'header'
            ),
            'enforce_min_fare' => array(
              'id'          => 'enforce_min_fare',
              'name'        => esc_html__('Enforce Minimum Fare?', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to enforce a minimum fare amount? If yes, all fares below the set amount will be rounded up to the set amount.', 'quickcab')
            ),
            'minimum_fare_amount' => array(
              'id'       => 'minimum_fare_amount',
              'name'     => esc_html__('Minimum Fare Amount', 'quickcab'),
              'type'     => 'number',
              'is_price' => true,
              'min'      => 0,
              'step'     => 0.01,
              'size'     => 'small',
              'prefix'   => get_woocommerce_currency_symbol(),
              'disabled' => false
            ),
            'enforce_max_fare' => array(
              'id'          => 'enforce_max_fare',
              'name'        => esc_html__('Enforce Maximum Fare?', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to enforce a maximum fare amount? If yes, all fares above the set amount will be rounded down to the set amount.', 'quickcab')
            ),
            'maximum_fare_amount' => array(
              'id'       => 'maximum_fare_amount',
              'name'     => esc_html__('Maximum Fare Amount', 'quickcab'),
              'type'     => 'number',
              'is_price' => true,
              'min'      => 0,
              'step'     => 0.01,
              'size'     => 'small',
              'prefix'   => get_woocommerce_currency_symbol(),
              'disabled' => false
            ),
            'min_max_distance_header' => array(
              'id'   => 'min_max_distance_header',
              'name' => '<h3>' . esc_html__('Min/Max Distance', 'quickcab') . '</h3>',
              'type' => 'header'
            ),
            'enforce_min_distance' => array(
              'id'          => 'enforce_min_distance',
              'name'        => esc_html__('Enforce Minimum Distance?', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to enforce a minimum distance? If yes, all fares below the set amount will produce the error message specified below.', 'quickcab')
            ),
            'minimum_fare_distance' => array(
              'id'       => 'minimum_fare_distance',
              'name'     => esc_html__('Minimum Fare Distance', 'quickcab'),
              'type'     => 'number',
              'is_price' => false,
              'min'      => 0,
              'size'     => 'small',
              'disabled' => false,
              'suffix'   => ' ' . $units
            ),
            'minimum_fare_distance_error_message' => array(
              'id'          => 'minimum_fare_distance_error_message',
              'name'        => esc_html__('Minimum Distance Error Message', 'quickcab'),
              'type'        => 'textarea',
              'placeholder' => quickcab_get_settings_defaults()['minimum_fare_distance_error_message']
            ),
            'enforce_max_distance' => array(
              'id'          => 'enforce_max_distance',
              'name'        => esc_html__('Enforce Maximum Distance?', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to enforce a maximum distance? If yes, all fares above the set amount will produce the error message specified below.', 'quickcab')
            ),
            'maximum_fare_distance' => array(
              'id'       => 'maximum_fare_distance',
              'name'     => esc_html__('Maximum Fare Distance', 'quickcab'),
              'type'     => 'number',
              'is_price' => false,
              'min'      => 0,
              'size'     => 'small',
              'suffix'   => ' ' . $units
            ),
            'maximum_fare_distance_error_message' => array(
              'id'       => 'maximum_fare_distance_error_message',
              'name'     => esc_html__('Maximum Distance Error Message', 'quickcab'),
              'type'     => 'textarea',
              'placeholder' => quickcab_get_settings_defaults()['maximum_fare_distance_error_message']
            )
          )
        )
      )
    )),
    'form' => apply_filters('quickcab_settings_form', array(
      'title' => esc_html__('Form', 'quickcab'),
      'sections' => array(
        'main' => array(
          'title' => esc_html__('Form Settings', 'quickcab'),
          'settings' => array(
            'form_settings_helper' => array(
              'id' => 'form_settings_helper',
              'name' => '',
              'type' => 'descriptive_text',
              'desc' => __('The settings listed below are global and will be used as defaults for Booking Forms. Changing settings for specific Booking Forms will override these settings.', 'quickcab')
            ),
            'form_primary_color' => array(
              'id'          => 'form_primary_color',
              'name'        => esc_html__('Form Primary Color', 'quickcab'),
              'type'        => 'colorpicker'
            ),
            'form_secondary_color' => array(
              'id'   => 'form_secondary_color',
              'name' => esc_html__('Form Secondary Color', 'quickcab'),
              'type' => 'colorpicker'
            ),
            'booking_time_interval_minutes' => array(
              'id'          => 'booking_time_interval_minutes',
              'name'        => esc_html__('Booking time interval (minutes)', 'quickcab'),
              'type'        => 'select',
              'options'     => array(
                '5' => '5',
                '10' => '10',
                '15' => '15',
                '30' => '30',
                '60' => '60',
              ),
              'helper_text' => esc_html__('The number of minutes between each booking time option in the form.', 'quickcab')
            ),
            // 'user_cant_chose_occupants' => array(
            //   'id'          => 'user_cant_chose_occupants',
            //   'name'        => esc_html__("Hide occupants option", 'quickcab'),
            //   'type'        => 'checkbox',
            //   'helper_text' => esc_html__('The user will not be able to select the number of occupants (defaults to 0).', 'quickcab')
            // ),
            // 'user_cant_chose_direction' => array(
            //   'id'          => 'user_cant_chose_direction',
            //   'name'        => esc_html__('One way only', 'quickcab'),
            //   'type'        => 'checkbox',
            //   'helper_text' => esc_html__('The user will not be able to select a return journey.', 'quickcab')
            // ),
            'use_fixed_addresses_starting' => array(
              'id'          => 'use_fixed_addresses_starting',
              'name'        => esc_html__('Used Fixed Addresses for Starting Address Field', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to use Fixed Addresses for the Starting Address field? You can create Fixed Addresses in Sidebar > QuickCab > Fixed Addresses.', 'quickcab')
            ),
            'use_fixed_addresses_destination' => array(
              'id'          => 'use_fixed_addresses_destination',
              'name'        => esc_html__('Used Fixed Addresses for Destination Address Field', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to use Fixed Addresses for the Destination Address field? You can create Fixed Addresses in Sidebar > QuickCab > Fixed Addresses.', 'quickcab')
            ),
            'enforce_autocompelte_country_restriction' => array(
              'id'          => 'enforce_autocompelte_country_restriction',
              'name'        => esc_html__('Enforce Autocomplete Country Restriction?', 'quickcab'),
              'type'        => 'checkbox',
              'helper_text' => esc_html__('Do you want to restrict the autocomplete values in the form to locations in a specific country?', 'quickcab')
            ),
            'autocomplete_country' => array(
              'id'          => 'autocomplete_country',
              'name'        => esc_html__('Autocomplete Country', 'quickcab'),
              'type'        => 'select',
              'options'     => quickcab_iso_3166_country_codes(),
              'placeholder' => esc_html__('Select a country', 'quickcab') . '...',
              'helper_text' => esc_html__('The google maps autocomplete on the booking form will be restricted to this country if the setting is on.', 'quickcab'),
              'dependency'  => array(
                'type' => 'is_checked',
                'id'   => 'enforce_autocompelte_country_restriction'
              )
            ),
            'no_quotes_error_message' => array(
              'id'          => 'no_quotes_error_message',
              'name'        => esc_html__('No Quotes Error Message', 'quickcab'),
              'type'        => 'textarea',
              'helper_text' => esc_html__('The error message that displays when no quotes are able to be returned for the journey and options the user has selected.', 'quickcab'),
              'placeholder' => quickcab_get_settings_defaults()['no_quotes_error_message']
            ),
          )
        ),
      )
    ))
  );

  return apply_filters('quickcab_registered_settings', $quickcab_settings);
}

function quickcab_no_callback($args) {
  printf(
    esc_html__('No callback for element with ID "%s" of type "%s". Please inform the developer of this error, quoting the ID.', 'quickcab'),
    '<strong>' . $args['id'] . '</strong>',
    '<strong>' . $args['type'] . '</strong>'
  );
}

/**
 * Render number field
 */
function quickcab_number_callback($args) {
  $value = quickcab_get_option($args['id'], false);

  if ( $args['is_price'] && $value ) {
    $value = round($value, 2);
  }

  $size = !is_null($args['size']) ? $args['size'] : 'regular';

  echo esc_html($args['prefix']);

  ?><input type="number" step="<?php echo esc_attr($args['step']); ?>" max="<?php echo esc_attr($args['max']); ?>" min="<?php echo esc_attr($args['min']); ?>" class="<?php echo sanitize_html_class($args['class']) . ' ' . sanitize_html_class($size) . '-text'; ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" value="<?php echo esc_attr(stripslashes($value)); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?> /><?php

  echo esc_html($args['suffix']);

  ?><p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render select field
 */
function quickcab_select_callback($args) {
  $value = quickcab_get_option($args['id'], false);

  if ( array_key_exists('type', $args['dependency']) && !quickcab_setting_should_display($args) ) {
    $args['disabled'] = true;
  }

  ?><select id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" class="<?php echo sanitize_html_class($args['class']); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?>><?php
    if ( !empty($args['placeholder']) ) {
      ?><option <?php if (empty($value)) { ?>selected<?php } ?> disabled><?php echo esc_html($args['placeholder']); ?></option><?php
    }

    foreach ( $args['options'] as $option => $name ) {
      ?><option value="<?php echo esc_attr($option); ?>" <?php echo selected($option, $value, false); ?>><?php
        echo esc_html($name);
      ?></option><?php
    }
  ?></select>

  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render select field
 */
function quickcab_radio_callback($args) {
  $value = quickcab_get_option($args['id']);

  foreach ( $args['options'] as $key => $name ) {
    ?><label for="quickcab_settings[<?php echo esc_attr($args['id']); ?>][<?php echo esc_attr($key); ?>]" class="quickcab-settings__radio-label">
      <input type="radio" value="<?php echo esc_attr($key); ?>" <?php checked($value, $key); ?> name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>][<?php echo esc_attr($key); ?>]"><?php
      echo esc_html($name);
    ?></label><?php
  }

  ?><p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render multiple checkbox field
 */
function quickcab_checkbox_multi_callback($args) {
  $value = quickcab_get_option($args['id'], false);

  foreach ( $args['options'] as $key => $option ) {
    $checked = isset($value[$key]);

    ?><label for="quickcab_settings[<?php echo esc_attr($args['id']); ?>][<?php echo esc_attr($key); ?>]">
      <input type="checkbox" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>][<?php echo esc_attr($key); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>][<?php echo esc_attr($key); ?>]" class="<?php echo sanitize_html_class($args['class']); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?> <?php checked($checked); ?>/>
      <?php echo esc_html($option); ?>
    </label><br><?php
  }

  ?><p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render text input field
 */
function quickcab_text_callback($args) {
  $value = quickcab_get_option($args['id'], false);
  $size = !is_null($args['size']) ? $args['size'] : 'regular';

  if ( array_key_exists('type', $args['dependency']) && !quickcab_setting_should_display($args) ) {
    $args['disabled'] = true;
  }

  ?><input type="text" class="<?php echo sanitize_html_class($args['class']) . ' ' . sanitize_html_class($size) . '-text'; ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" value="<?php echo esc_attr(stripslashes($value)); ?>" placeholder="<?php echo esc_attr($args['placeholder']); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?> />
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

function quickcab_colorpicker_callback($args) {
  $value = quickcab_get_option($args['id']);

  ?><input type="text" class="<?php echo sanitize_html_class($args['class']); ?> quickcab-color-picker" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" value="<?php echo esc_attr(stripslashes($value)); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?> />
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render email input field
 */
function quickcab_email_callback($args) {
  $value = quickcab_get_option($args['id'], false);
  $size = !is_null($args['size']) ? $args['size'] : 'regular';

  ?><input type="email" class="<?php echo sanitize_html_class($args['class']) . ' ' . sanitize_html_class($size) . '-text'; ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" value="<?php echo esc_attr(stripslashes($value)); ?>" placeholder="<?php echo esc_attr($args['placeholder']); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?> />
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render checkbox field
 */
function quickcab_checkbox_callback($args) {
  global $quickcab_settings;
  $value = quickcab_get_option($args['id'], false);

  ?><input type="checkbox" class="<?php echo sanitize_html_class($args['class']); ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" <?php checked($value == 'on'); ?> <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?>>
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render textarea field
 */
function quickcab_textarea_callback($args) {
  $value = quickcab_get_option($args['id'], false);

  ?><textarea class="<?php echo sanitize_html_class($args['class']); ?> large-text" cols="<?php echo esc_attr($args['cols']); ?>" rows="<?php echo esc_attr($args['rows']); ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" placeholder="<?php echo esc_attr($args['placeholder']); ?>"><?php
    echo esc_textarea(stripslashes($value));
  ?></textarea>
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

/**
 * Render header
 */
function quickcab_header_callback($args) {
  echo '';
}

/**
 * Render descriptive text
 */
function quickcab_descriptive_text_callback($args) {
  echo wp_kses_post($args['desc']);
}

/**
 * Render google autocomplete field
 */
function quickcab_google_autocomplete_callback($args) {
  $value = quickcab_get_option($args['id'], false);
  $size = !is_null($args['size']) ? $args['size'] : 'regular';

  ?><input type="text" class="<?php echo sanitize_html_class($args['class']) . ' ' . sanitize_html_class($size) . '-text'; ?> quickcab-google-autocomplete-field" value="<?php echo esc_attr(stripslashes($value)); ?>" placeholder="<?php echo esc_attr($args['placeholder']); ?>" <?php echo !is_null($args['disabled']) && $args['disabled'] ? 'disabled' : ''; ?>>
  <input type="hidden" value="<?php echo esc_attr(stripslashes($value)); ?>" id="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" name="quickcab_settings[<?php echo esc_attr($args['id']); ?>]" class="quickcab-hidden-autocomplete-field">
  <span class="dashicons dashicons-yes quickcab-autocomplete-tick <?php if ($value) { echo '-green'; } ?>"></span>
  <p class="description" id="<?php echo esc_attr($args['helper_id']); ?>"><?php
    echo wp_kses_post($args['helper_text']);
  ?></p><?php
}

function quickcab_setting_should_display($args) {
  $type = $args['dependency']['type'];
  $id = $args['dependency']['id'];

  switch ($type) {
    case 'is_checked':
      return quickcab_get_option($id) == 'on';
    default:
      return true;
  }
}

function quickcab_get_settings() {
  $settings = get_option('quickcab_settings');

  if ( empty($settings) ) {
    $settings = array();
  }

  return apply_filters('quickcab_get_settings', $settings);
}

/**
 * Get a QuickCab option, looks to see if the specified setting exists, returns
 * default if not
 *
 * @since  1.0
 * @param  string  $key           the name of the setting
 * @param  boolean $give_default  provide the default value?
 * @return string                 the value of the option
 */
function quickcab_get_option($key, $give_default = true) {
  global $quickcab_settings;

  $defaults = quickcab_get_settings_defaults();
  $default = isset($defaults[$key]) && $give_default ? $defaults[$key] : null;

  $value = empty($quickcab_settings[$key]) ? $default : $quickcab_settings[$key];
  $value = apply_filters('quickcab_get_option', $value, $key, $default);

  return apply_filters('quickcab_get_option_' . $key, $value, $default);
}

/**
 * Set a QuickCab option and save in global variable
 *
 * @since  1.0
 * @param  string     $key    option key
 * @param  boolean    $value  option value
 * @return boolean            did the option save
 */
function quickcab_set_option($key = '', $value = false) {
	if ( empty($key) ) {
		return false;
	}

	if ( empty($value) ) {
		$remove_option = quickcab_delete_option($key);
		return $remove_option;
	}

	$options = quickcab_get_settings();

	$value = apply_filters('quickcab_update_option', $value, $key);

	$options[$key] = $value;
	$did_update = update_option('quickcab_settings', $options);

	if ( $did_update ) {
		global $quickcab_settings;
		$quickcab_settings[$key] = $value;
	}

	return $did_update;
}

/**
 * Delete a QuickCab option
 *
 * @since   1.0
 * @param   string     $key  the option key
 * @return  boolean          did the option delete
 */
function quickcab_delete_option($key = '') {
	global $quickcab_settings;

	if ( empty($key) ) {
		return false;
	}

  $options = quickcab_get_settings();

	if ( isset($options[$key]) ) {
		unset($options[$key]);
	}

	if ( isset($quickcab_settings[$key]) ) {
		unset($quickcab_settings[$key] );
	}

	$did_update = update_option('quickcab_settings', $options);

	if ( $did_update ) {
		global $quickcab_settings;
		$quickcab_settings = $options;
	}

	return $did_update;
}

/**
 * Sanitize the settings saved by the user
 *
 * @since   1.0
 * @param   array      $input  the array of settings the user has saved
 * @return  array              the sanitized array of options
 */
function quickcab_settings_sanitize($input = array()) {
  global $quickcab_settings;

  $doing_section = !empty(wp_unslash($_POST['_wp_http_referer']));
  $setting_types = quickcab_get_registered_settings_types();
  $input = $input ? $input : array();

  if ( $doing_section ) {
    parse_str(wp_unslash($_POST['_wp_http_referer']), $referrer);
    $tab = isset($referrer['tab']) ? $referrer['tab'] : 'general';
    $section = isset($referrer['section']) ? $referrer['section'] : 'main';

    if ( !empty(wp_unslash($_POST['quickcab_section_override'])) ) {
      $section = sanitize_text_field(wp_unslash($_POST['quickcab_section_override']));
    }

    $setting_types = quickcab_get_registered_settings_types($tab, $section);

    $input = apply_filters('quickcab_settings_' . $tab . '_sanitize', $input);
    $input = apply_filters('quickcab_settings_' . $tab . '-' . $section . '_sanitize', $input);
  }

  $output = array_merge($quickcab_settings, $input);

  $non_setting_types = apply_filters('quickcab_non_setting_types', array(
    'header', 'descriptive_text'
  ));

  foreach ( $setting_types as $key => $type ) {

    if ( empty($type) || in_array($type, $non_setting_types) ) {
      continue;
    }

    if ( array_key_exists($key, $output) ) {
      $output[$key] = apply_filters('quickcab_settings_sanitize_' . $type, $output[$key], $key);
      $output[$key] = apply_filters('quickcab_settings_sanitize', $output[$key], $key);
    }

    if ( $doing_section ) {
      switch ( $type ) {
        case 'checkbox':
        case 'gateways':
        case 'multicheck':
        case 'payment_icons':
          if ( !array_key_exists($key, $input) && array_key_exists($key, $setting_types) ) {
            unset($output[$key]);
          }
          break;
        default:
          if ( (array_key_exists($key, $input) && empty($input[$key])) || (array_key_exists($key, $output) && !array_key_exists($key, $input)) ) {
            unset($output[$key]);
          }
          break;
      }
    } else {
      if ( empty($input[$key]) ) {
        unset($output[$key]);
      }
    }

  }

  if ( $doing_section ) {
    add_settings_error('quickcab-notices', '', esc_html__('Settings updated.', 'quickcab'), 'updated');
  }

  return $output;
}

/**
 * Get the types of all settings
 *
 * @since  1.0
 * @return array  associative array of setting types
 */
function quickcab_get_registered_settings_types($filtered_tab = false, $filtered_section = false) {
  $setting_types = array();

  foreach ( quickcab_get_registered_settings() as $tab_id => $tab ) {
    if ( $filtered_tab !== false && $filtered_tab !== $tab_id ) {
      continue;
    }

    foreach ( $tab['sections'] as $section_id => $section ) {
      if ( $filtered_section !== false && $filtered_section !== $section_id ) {
        continue;
      }

      foreach ( $section['settings'] as $setting_id => $setting ) {
        $setting_types[$setting['id']] = $setting['type'];
      }
    }
  }

  return $setting_types;
}

/**
 * Get defaults of settings, must be stored in separate array to prevent
 * infinite loop
 *
 * @since  1.0
 * @return array  array of defaults
 */
function quickcab_get_settings_defaults() {
  $defaults = array(
    'distance_units'                             => 'miles',
    'currency'                                   => 'USD',
    'thousands_separator'                        => ',',
    'decimal_separator'                          => '.',
    'decimal_points_shown'                       => 2,
    'return_discount_rate'                       => 0,
    'minimum_fare_distance'                      => 0,
    'minimum_fare_distance_error_message'        => esc_html__('Sorry, this journey is too short.', 'quickcab'),
    'maximum_fare_distance_error_message'        => esc_html__('Sorry, this journey is too long.', 'quickcab'),
    'form_heading'                               => esc_html__('Book a Ride!', 'quickcab'),
    'direction_label'                            => esc_html__('Direction', 'quickcab'),
    'passengers_label'                           => esc_html__('Passengers', 'quickcab'),
    'suitcases_label'                            => esc_html__('Suitcases', 'quickcab'),
    'departure_date_label'                       => esc_html__('Departure Date', 'quickcab'),
    'departure_time_label'                       => esc_html__('Departure Time', 'quickcab'),
    'return_departure_date_label'                => esc_html__('Return Departure Date', 'quickcab'),
    'return_departure_time_label'                => esc_html__('Return Departure Time', 'quickcab'),
    'choose_vehicle_button_text'                 => esc_html__('Choose Vehicle', 'quickcab'),
    'book_vehicle_button_text'                   => esc_html__('Book Now', 'quickcab'),
    'billing_information_header'                 => esc_html__('Billing Information', 'quickcab'),
    'got_to_payment_button_text'                 => esc_html__('Go To Payment', 'quickcab'),
    'place_order_button_text'                    => esc_html__('Pay Now', 'quickcab'),
    'no_quotes_error_message'                    => esc_html__('Sorry there are no vehicles available for the options you have entered.', 'quickcab'),
    'price_breakdown_base_fare_label'            => esc_html__('Base Fare', 'quickcab'),
    'price_breakdown_passengers_label'           => esc_html__('Passengers', 'quickcab'),
    'price_breakdown_suitcases_label'            => esc_html__('Suitcases', 'quickcab'),
    'price_breakdown_distance_label'             => esc_html__('Distance', 'quickcab'),
    'price_breakdown_duration_label'             => esc_html__('Duration', 'quickcab'),
    'price_breakdown_return_fare_discount_label' => esc_html__('Return Fare Discount', 'quickcab'),
    'price_breakdown_min_fare_surcharge_label'   => esc_html__('Min Fare Surcharge', 'quickcab'),
    'price_breakdown_max_fare_discount_label'    => esc_html__('Max Fare Discount', 'quickcab'),
    'form_primary_color'                         => '#1080f2',
    'form_secondary_color'                       => '#fff',
    'default_calendar_period'                    => 'dayGridMonth',
    'booking_time_interval_minutes'              => '5'
  );

  return apply_filters('quickcab_settings_defaults', $defaults);
}
