<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_surge_price_meta_box() {
  add_meta_box('quickcab_surge_prices_date_options', esc_html__('Date/Time Options', 'quickcab'),  'quickcab_render_surge_price_date_meta_box', array('surge_price_rule'), 'normal', 'high');

  add_meta_box('quickcab_surge_prices_price_options', esc_html__('Price Options', 'quickcab'),  'quickcab_render_surge_price_pricing_meta_box', array('surge_price_rule'), 'normal', 'default');
}
add_action('add_meta_boxes', 'quickcab_add_surge_price_meta_box');

function quickcab_render_surge_price_date_meta_box() {
  global $post;

  do_action('quickcab_meta_box_surge_price_date_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_surge_pricing_meta_box_nonce');
}

function quickcab_render_surge_price_pricing_meta_box() {
  global $post;

  do_action('quickcab_meta_box_surge_price_pricing_fields', $post->ID);
}

function quickcab_render_surge_price_date_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);
  $d_struct = $surge_price_rule->get_date_structure();

  $hide_fixed_date_fields = apply_filters('quickcab_surge_price_hide_fixed_date_fields', array(
    'days-of-week'
  ));

  $show_days_of_week_fields = apply_filters('quickcab_surge_price_show_days_of_week_fields', array(
    'days-of-week'
  ));

  $days = array(
    1 => esc_html__('Monday', 'quickcab'),
    2 => esc_html__('Tuesday', 'quickcab'),
    3 => esc_html__('Wednesday', 'quickcab'),
    4 => esc_html__('Thursday', 'quickcab'),
    5 => esc_html__('Friday', 'quickcab'),
    6 => esc_html__('Saturday', 'quickcab'),
    0 => esc_html__('Sunday', 'quickcab')
  );

  ?><table class="form-table">
    <tbody>
      <tr>
        <th scope="row">
          <label for="quickcab_vehicle_pricing_structure"><?php
            echo esc_html__('Dates applied', 'quickcab') . ':';
          ?></label>
        </th>
        <td>
          <select name="quickcab_surge_price_date_structure" id="quickcab_surge_price_date_structure"><?php

            $pricing_options = apply_filters('quickcab_surge_price_date_structure_options', array(
              'fixed-date'   => esc_html__('Fixed Date', 'quickcab'),
              'days-of-week' => esc_html__('Days of Week', 'quickcab')
            ));

            foreach ( $pricing_options as $key => $pricing_option ) {
              ?><option value="<?php echo esc_attr($key); ?>" <?php if ($d_struct == $key) { ?>selected="selected"<?php } ?>><?php
                echo esc_html($pricing_option);
              ?></option><?php
            }

          ?></select>
        </td>
      </tr>
    </tbody>

    <tbody <?php if (in_array($d_struct, $hide_fixed_date_fields)) { ?>style="display: none;"<?php } ?>id="quickcab_surge_price_fixed_date_section">
      <tr>
        <th scope="row">
          <label for="quickcab_vehicle_pricing_structure"><?php
            echo esc_html__('Start date', 'quickcab') . ':';
          ?></label>
        </th>
        <td>
          <input type="text" name="quickcab_surge_price_fixed_start_date" id="quickcab_surge_price_fixed_start_date" <?php if (!$is_new) { ?>value="<?php echo esc_attr($surge_price_rule->get_fixed_start_date()); ?>"<?php } ?>>
          <select name="quickcab_surge_price_fixed_start_time_hour" id="quickcab_surge_price_fixed_start_time_hour"><?php
            for ( $i = 0; $i <= 23; $i++ ) {
              ?><option value="<?php echo sprintf('%02d', $i); ?>" <?php if (!$is_new && $surge_price_rule->get_fixed_start_time_hour() == $i) { ?>selected<?php } ?>><?php
                echo sprintf('%02d', $i);
              ?></option><?php
            }
          ?></select>
          :
          <select name="quickcab_surge_price_fixed_start_time_minute" id="quickcab_surge_price_fixed_start_time_minute"><?php
            for ( $i = 0; $i <= 59; $i = $i + 5 ) {
              ?><option value="<?php echo sprintf('%02d', $i); ?>" <?php if (!$is_new && $surge_price_rule->get_fixed_start_time_minute() == $i) { ?>selected<?php } ?>><?php
                echo sprintf('%02d', $i);
              ?></option><?php
            }
          ?></select>
        </td>
      </tr>

      <tr>
        <th scope="row">
          <label for="quickcab_vehicle_pricing_structure"><?php
            echo esc_html__('End date', 'quickcab') . ':';
          ?></label>
        </th>
        <td>
          <input type="text" name="quickcab_surge_price_fixed_end_date" id="quickcab_surge_price_fixed_end_date" <?php if (!$is_new) { ?>value="<?php echo esc_attr($surge_price_rule->get_fixed_end_date()); ?>"<?php } ?>>
          <select name="quickcab_surge_price_fixed_end_time_hour" id="quickcab_surge_price_fixed_end_time_hour"><?php
            for ( $i = 0; $i <= 23; $i++ ) {
              ?><option value="<?php echo sprintf('%02d', $i); ?>" <?php if (!$is_new && $surge_price_rule->get_fixed_end_time_hour() == $i) { ?>selected<?php } ?>><?php
                echo sprintf('%02d', $i);
              ?></option><?php
            }
          ?></select>
          :
          <select name="quickcab_surge_price_fixed_end_time_minute" id="quickcab_surge_price_fixed_end_time_minute"><?php
            for ( $i = 0; $i <= 59; $i = $i + 5 ) {
              ?><option value="<?php echo sprintf('%02d', $i); ?>" <?php if (!$is_new && $surge_price_rule->get_fixed_end_time_minute() == $i) { ?>selected<?php } ?>><?php
                echo sprintf('%02d', $i);
              ?></option><?php
            }
          ?></select>
        </td>
      </tr>
    </tbody>

    <tbody <?php if ($is_new || !in_array($d_struct, $show_days_of_week_fields)) { ?>style="display: none;"<?php } ?> id="quickcab_surge_price_day_of_week_section">
      <tr>
        <th scope="row">
          <label for="quickcab_vehicle_pricing_structure"><?php
            echo esc_html__('Time of condition', 'quickcab') . ':';
          ?></label>
        </th>
        <td>
          <div class="quickcab-surge-pricing__day-block">
            <select name="quickcab_surge_price_day_start_day" id="quickcab_surge_price_day_start_day"><?php
              foreach ( $days as $i => $day ) {
                ?><option value="<?php echo esc_attr($i); ?>"><?php
                  echo esc_html($day);
                ?></option><?php
              }
            ?></select>
            <?php echo esc_html__('at', 'quickcab'); ?>
            <select name="quickcab_surge_price_day_start_time_hour" id="quickcab_surge_price_day_start_time_hour"><?php
              for ( $i = 0; $i <= 23; $i++ ) {
                ?><option value="<?php echo sprintf('%02d', $i); ?>"><?php
                  echo sprintf('%02d', $i);
                ?></option><?php
              }
            ?></select>
            :
            <select name="quickcab_surge_price_day_start_time_minute" id="quickcab_surge_price_day_start_time_minute"><?php
              for ( $i = 0; $i <= 59; $i = $i + 5 ) {
                ?><option value="<?php echo sprintf('%02d', $i); ?>"><?php
                  echo sprintf('%02d', $i);
                ?></option><?php
              }
            ?></select>
          </div>
          <div class="quickcab-surge-pricing__day-block">
            &mdash;
          </div>
          <div class="quickcab-surge-pricing__day-block">
            <select name="quickcab_surge_price_day_end_day" id="quickcab_surge_price_day_end_day"><?php
              foreach ( $days as $i => $day ) {
                ?><option value="<?php echo esc_attr($i); ?>"><?php
                  echo esc_html($day);
                ?></option><?php
              }
            ?></select>
            <?php echo esc_html__('at', 'quickcab'); ?>
            <select name="quickcab_surge_price_day_end_time_hour" id="quickcab_surge_price_day_end_time_hour"><?php
              for ( $i = 0; $i <= 23; $i++ ) {
                ?><option value="<?php echo sprintf('%02d', $i); ?>"><?php
                  echo sprintf('%02d', $i);
                ?></option><?php
              }
            ?></select>
            :
            <select name="quickcab_surge_price_day_end_time_minute" id="quickcab_surge_price_day_end_time_minute"><?php
              for ( $i = 0; $i <= 59; $i = $i + 5 ) {
                ?><option value="<?php echo sprintf('%02d', $i); ?>"><?php
                  echo sprintf('%02d', $i);
                ?></option><?php
              }
            ?></select>
          </div>
          <div class="quickcab-surge-pricing__day-block">
            <a href="#" class="button button-primary" id="quickcab_surge_price_day_add_button"><?php
              echo esc_html__('Add time', 'quickcab');
            ?></a>
          </div>
          <table>
            <thead>
              <tr>
                <th><?php
                  echo esc_html__('From', 'quickcab');
                ?></th>
                <th><?php
                  echo esc_html__('To', 'quickcab');
                ?></th>
                <th><?php
                  echo esc_html__('Actions', 'quickcab');
                ?></th>
              </tr>
            </thead>
            <tbody id="quickcab_surge_price_table_body"><?php
              foreach ( $surge_price_rule->get_day_tiers() as $i => $time_tier ) {
                ?><tr>
                  <td><?php
                    echo esc_html($days[$time_tier['from_day']]) . ' ' . esc_html__('at', 'quickcab') . ' ' . sprintf('%02d', $time_tier['from_hour']) . ':' . sprintf('%02d', $time_tier['from_min']);
                  ?></td>
                  <td><?php
                    echo esc_html($days[$time_tier['to_day']]) . ' ' . esc_html__('at', 'quickcab') . ' ' . sprintf('%02d', $time_tier['to_hour']) . ':' . sprintf('%02d', $time_tier['to_min']);
                  ?></td>
                  <td>
                    <a href="#" class="button button-default quickcab-surge-price__remove-time-row"><?php
                      echo esc_html__('Remove time', 'quickcab');
                    ?></a>
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][from_day]" value="<?php echo esc_html($time_tier['from_day']); ?>">
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][from_hour]" value="<?php echo esc_html($time_tier['from_hour']); ?>">
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][from_min]" value="<?php echo esc_html($time_tier['from_min']); ?>">
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][to_day]" value="<?php echo esc_html($time_tier['to_day']); ?>">
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][to_hour]" value="<?php echo esc_html($time_tier['to_hour']); ?>">
                    <input type="hidden" name="quickcab_surge_price_day_tiers[<?php echo esc_attr($i); ?>][to_min]" value="<?php echo esc_html($time_tier['to_min']); ?>">
                  </td>
                </tr><?php
              }
            ?></tbody>
          </table>
          <p class="description"><?php
            echo esc_html__('You can add multiple time brackets here, this conditon will apply for all of them.', 'quickcab');
          ?></p>
        </td>
      </tr>
    </tbody>
  </table><?php
}
add_action('quickcab_meta_box_surge_price_date_fields', 'quickcab_render_surge_price_date_fields', 10);

function quickcab_render_surge_price_pricing_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);
  $p_struct = $surge_price_rule->get_pricing_structure();

  $hide_increase_amount_field = apply_filters('quickcab_surge_price_hide_increase_amount', array(
    'increase-multiplier'
  ));

  $show_increase_multiplier_field = apply_filters('quickcab_surge_price_show_increase_multiplier', array(
    'increase-multiplier'
  ));

  ?><table class="form-table">
    <tbody>
      <tr>
        <th scope="row">
          <label for="quickcab_vehicle_pricing_structure"><?php
            echo esc_html__('Price increase', 'quickcab') . ':';
          ?></label>
        </th>
        <td>
          <select name="quickcab_surge_price_pricing_structure" id="quickcab_surge_price_pricing_structure"><?php

            $pricing_options = apply_filters('quickcab_surge_price_pricing_structure_options', array(
              'increase-amount'     => esc_html__('Increase by Amount', 'quickcab'),
              'increase-multiplier' => esc_html__('Increase by Multiplier', 'quickcab')
            ));

            foreach ( $pricing_options as $key => $pricing_option ) {
              ?><option value="<?php echo esc_attr($key); ?>" <?php if ($p_struct == $key) { ?>selected="selected"<?php } ?>><?php
                echo esc_html($pricing_option);
              ?></option><?php
            }

          ?></select>
        </td>
      </tr>
    </tbody>

    <tbody id="quickcab_surge_price_increase_amount" <?php if (in_array($p_struct, $hide_increase_amount_field)) { ?>style="display:none;"<?php } ?>><?php
      quickcab_render_surge_price_metabox_price_row(
        'quickcab_surge_price_increase_amount',
        $post_id,
        array(
          'text' => esc_html__('Increase amount', 'quickcab') . ':'
        ),
        array(
          'min'    => 0,
          'prefix' => '+'
        )
      );
    ?></tbody>

    <tbody id="quickcab_surge_price_increase_multiplier" <?php if ($is_new || !in_array($p_struct, $show_increase_multiplier_field)) { ?>style="display:none;"<?php } ?>><?php
      quickcab_render_surge_price_metabox_number_row(
        'quickcab_surge_price_increase_multiplier',
        $post_id,
        array(
          'text' => esc_html__('Increase multiplier', 'quickcab') . ':'
        ),
        array(
          'min'    => 0,
          'prefix' => 'x',
          'step' => '0.0001'
        )
      );
    ?></tbody>
  </table><?php
}
add_action('quickcab_meta_box_surge_price_pricing_fields', 'quickcab_render_surge_price_pricing_fields', 10);

/**
 * The fields in the metaboxes that should be saved
 *
 * @since  1.0
 * @return array  array of field names to be saved
 */
function quickcab_surge_price_metabox_fields() {
  return apply_filters('quickcab_surge_price_metabox_fields_save', array(
    'quickcab_surge_price_date_structure',
    'quickcab_surge_price_fixed_start_date',
    'quickcab_surge_price_fixed_start_time_hour',
    'quickcab_surge_price_fixed_start_time_minute',
    'quickcab_surge_price_fixed_end_date',
    'quickcab_surge_price_fixed_end_time_hour',
    'quickcab_surge_price_fixed_end_time_minute',
    'quickcab_surge_price_day_tiers',
    'quickcab_surge_price_pricing_structure',
    'quickcab_surge_price_increase_amount',
    'quickcab_surge_price_increase_multiplier'
  ));
}

/**
 * Run after the post is saved, saves all data in metaboxes
 *
 * @since  1.0
 * @param  int     $post_id  ID of the post
 * @param  WP_POST $post     the post being saved
 * @return void
 */
function quickcab_surge_price_meta_box_save($post_id, $post) {
  if ( !isset($_POST['quickcab_surge_pricing_meta_box_nonce']) || !wp_verify_nonce(wp_unslash($_POST['quickcab_surge_pricing_meta_box_nonce']), basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $surge_price_rule = new QuickCab_Surge_Price_Rule($post_id);

  foreach ( quickcab_surge_price_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_surge_price_', '', $field);

    if ( empty($_POST[$field]) ) {
      $surge_price_rule->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_surge_price_metabox_save_' . $meta_name, wp_unslash($_POST[$field]), $post_id);
      $surge_price_rule->update_meta($meta_name, $new);
    }
  }

  do_action('quickcab_save_surge_price_rule', $post_id, $post);
}
add_action('save_post', 'quickcab_surge_price_meta_box_save', 10, 2);

/**
 * Serialise the days to apply this condition to for easy storage
 *
 * @since  1.0
 * @param  array  $tiers  array of tiers
 * @return string         the serialised string
 */
function quickcab_surge_price_serlialise_day_tiers($tiers, $post_id) {
  return serialize($tiers);
}
add_filter('quickcab_surge_price_metabox_save_day_tiers', 'quickcab_surge_price_serlialise_day_tiers', 10, 2);
