<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_vehicle_columns($vehicle_columns) {
  $taxonomy = get_taxonomy('vehicle_features');

  $vehicle_columns = array(
    'cb'                => '<input type="checkbox"/>',
    'image'             => esc_html__('Image', 'quickcab'),
    'title'             => esc_html__('Name', 'quickcab'),
    'number_owned'      => esc_html__('Number Owned', 'quickcab'),
    'passengers'        => esc_html__('Passengers', 'quickcab'),
    'suitcases'         => esc_html__('Suitcases', 'quickcab'),
    'price'             => esc_html__('Price', 'quickcab'),
    'vehicle_features'  => $taxonomy->labels->menu_name
  );

  return apply_filters('quickcab_vehicle_columns', $vehicle_columns);
}
add_filter('manage_quickcab_vehicle_posts_columns', 'quickcab_vehicle_columns');

function quickcab_vehicle_sortable_columns($columns) {
  $columns['number_owned'] = 'number_owned';
  $columns['passengers'] = 'passengers';
  $columns['suitcases'] = 'suitcases';

  return $columns;
}
add_filter('manage_edit-quickcab_vehicle_sortable_columns', 'quickcab_vehicle_sortable_columns');

function quickcab_render_vehicle_columns($column_name, $post_id) {
  if ( get_post_type($post_id) == 'quickcab_vehicle' ) {
    $vehicle = new QuickCab_Vehicle($post_id);

    switch ( $column_name ) {
      case 'image':
        ?><a href="<?php echo get_edit_post_link($post_id);?>">
          <img src="<?php echo esc_attr($vehicle->get_image_url('small-thumb')); ?>" class="quickcab-vehicles-index__image">
        </a><?php
        break;
      case 'vehicle_features':
        echo get_the_term_list($post_id, 'vehicle_features', '', ', ', '');
        break;
      case 'price':
        $singular_units = quickcab_singular_units_word(quickcab_get_option('distance_units'));

        switch ( $vehicle->get_pricing_structure() ) {
          case 'per-distance':

            ?><div class="qc-vehicle-table-starting-price"><?php
              echo esc_html__('Starting Price', 'quickcab'); ?>:
              <?php echo quickcab_format_price(esc_html($vehicle->get_starting_price())); ?>
            </div><?php

            echo quickcab_format_price(esc_html($vehicle->get_price_per_distance())) . " / " . esc_html($singular_units); ?><br /><?php
            echo quickcab_format_price(esc_html($vehicle->get_price_per_minute())) . " / " . esc_html__('minute', 'quickcab'); ?><br /><?php
            echo quickcab_format_price(esc_html($vehicle->get_price_per_occupant())) . " / " . esc_html__('occupant', 'quickcab'); ?><br /><?php
            echo quickcab_format_price(esc_html($vehicle->get_price_per_suitcase())) . " / " . esc_html__('suitcase', 'quickcab'); ?><br /><?php
            break;
          case 'per-hour':
            echo quickcab_format_price($vehicle->get_price_per_hour()) . " / " . esc_html__('hour', 'quickcab');
            break;
          case 'per-distance-tiered':
            ?><div class="qc-vehicle-table-starting-price">
              <strong><?php echo esc_html__('Starting Price', 'quickcab'); ?>:</strong>

              <?php echo quickcab_format_price(esc_html($vehicle->get_starting_price())); ?>
            </div><?php

            $distance_tiers = $vehicle->get_price_per_distance_tiered();

            if ( count($distance_tiers) > 0 ) {
              ?><table class="qc-vehicle-index-table-pricing-tiers">
                <thead>
                  <tr>
                    <th><?php echo esc_html__('Distance', 'quickcab'); ?></th>
                    <th><?php
                      echo apply_filters('quickcab_tiered_pricing_distance_price_header', sprintf(
                        __('Price per %s', 'quickcab'),
                        $singular_units
                      ));
                    ?></th>
                  </tr>
                </thead>
                <tbody><?php
                  foreach ( $distance_tiers as $distance_tier ) {
                    ?><tr>
                      <td><?php echo esc_html($distance_tier['range']); ?></td>
                      <td><?php
                        echo quickcab_format_price(esc_html($distance_tier['price']));
                        do_action('quickcab_tiered_pricing_distance_after_price', $distance_tier);
                      ?></td>
                    </tr><?php
                  }

                  ?><tr>
                    <td><?php
                      echo ($distance_tiers[count($distance_tiers) - 1]['value_to'] + 1) . '+';
                    ?></td>
                    <td><?php
                      echo quickcab_format_price(esc_html($vehicle->get_price_per_distance()));
                      do_action('quickcab_tiered_pricing_distance_after_price', null);
                    ?></td>
                  </tr>
                </tbody>
              </table><?php
            }

            echo quickcab_format_price(esc_html($vehicle->get_price_per_minute())) . " / " . esc_html__('minute', 'quickcab'); ?><br /><?php
            echo quickcab_format_price(esc_html($vehicle->get_price_per_occupant())) . " / " . esc_html__('occupant', 'quickcab'); ?><br /><?php
            echo quickcab_format_price(esc_html($vehicle->get_price_per_suitcase())) . " / " . esc_html__('suitcase', 'quickcab'); ?><br /><?php
            break;
          case 'per-hour-tiered':
            $hourly_tiers = $vehicle->get_price_per_hour_tiered();

            if ( count($hourly_tiers) > 0 ) {
              ?><table class="qc-vehicle-index-table-pricing-tiers">
                <thead>
                  <tr>
                    <th><?php echo esc_html__('Hours', 'quickcab'); ?></th>
                    <th><?php echo esc_html__('Price per hour:', 'quickcab'); ?></th>
                  </tr>
                </thead>

                <tbody><?php
                  foreach ( $hourly_tiers as $hourly_tier ) {
                    ?><tr>
                      <td><?php echo esc_html($hourly_tier['range']); ?></td>
                      <td><?php echo quickcab_format_price(esc_html($hourly_tier['price'])); ?></td>
                    </tr><?php
                  }

                  ?><tr>
                    <td><?php
                      echo ($hourly_tiers[count($hourly_tiers) - 1]['value_to'] + 1) . '+';
                    ?></td>
                    <td><?php
                      echo quickcab_format_price(esc_html($vehicle->get_price_per_hour()));
                    ?></td>
                  </tr>
                </tbody>
              </table><?php
            }
            break;
        }
        break;
      case 'number_owned':
        echo esc_html($vehicle->get_number_owned());
        break;
      case 'passengers':
        echo esc_html($vehicle->get_occupant_space());
        break;
      case 'suitcases':
        echo esc_html($vehicle->get_suitcase_space());
        break;
    }
  }
}
add_action('manage_posts_custom_column', 'quickcab_render_vehicle_columns', 10, 2);
