<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_add_vehicle_meta_box() {
  add_meta_box('quickcab_vehicle_prices', esc_html__('Vehicle Pricing', 'quickcab'),  'quickcab_render_vehicle_pricing_meta_box', array('quickcab_vehicle'), 'normal', 'high');
  add_meta_box('quickcab_vehicle_options', esc_html__('Vehicle Options', 'quickcab'),  'quickcab_render_vehicle_options_meta_box', array('quickcab_vehicle'), 'normal', 'default');
}
add_action('add_meta_boxes', 'quickcab_add_vehicle_meta_box');

function quickcab_render_vehicle_pricing_meta_box() {
  global $post;

  do_action('quickcab_meta_box_vehicle_price_fields', $post->ID);

  wp_nonce_field(basename(__FILE__), 'quickcab_vehicle_meta_box_nonce');
}

function quickcab_render_vehicle_options_meta_box() {
  global $post;

  do_action('quickcab_meta_box_vehicle_option_fields', $post->ID);
}

function quickcab_render_vehicle_price_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $vehicle = new QuickCab_Vehicle($post_id);

  $p_struct = $vehicle->get_pricing_structure();
  $units = quickcab_get_option('distance_units');

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_vehicle_metabox_select_row(
        'quickcab_vehicle_pricing_structure',
        $post_id,
        array(
          'text' => esc_html__('Pricing Structure', 'quickcab') . ':'
        ),
        array(
          'options' => apply_filters('quickcab_vehicle_pricing_options', array(
            'per-distance' => sprintf(
              esc_html__('Per %s', 'quickcab'),
              quickcab_singular_units_word($units)
            ),
            'per-hour' => esc_html__('Per hour', 'quickcab')
          ))
        )
      );
    ?></tbody><?php

    $hide_starting_fare_options = apply_filters('quickcab_vehicle_pricing_options_hide_starting_price', array(
      'per-hour'
    ));

    $hide_hourly_rate_options = apply_filters('quickcab_vehicle_pricing_options_hide_hourly_rate', array(
      'per-distance'
    ));

    ?><tbody id="quickcab_modular_pricing_options" <?php if (in_array($p_struct, $hide_starting_fare_options)) { ?>style="display:none;"<?php } ?>><?php
      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_starting_price',
        $post_id,
        array(
          'text' => esc_html__('Starting Price', 'quickcab') . ':'
        )
      );
    ?></tbody>
    <tbody id="quickcab_modular_pricing_fixed_options" <?php if (in_array($p_struct, $hide_starting_fare_options)) { ?>style="display:none;"<?php } ?>><?php
      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_distance',
        $post_id,
        array(
          'text' => sprintf(
            esc_html__('Price per %s', 'quickcab') . ':',
            quickcab_singular_units_word($units)
          )
        ),
        array(),
        apply_filters('quickcab_price_per_mile_helper_text', array(), $p_struct)
      );
    ?></tbody><?php

    do_action('quickcab_vehicle_metabox_after_price_per_distance', $post_id);

    ?><tbody id="quickcab_modular_pricing_options_part_2" <?php if (in_array($p_struct, $hide_starting_fare_options)) { ?>style="display:none;"<?php } ?>><?php
      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_minute',
        $post_id,
        array(
          'text' => esc_html__('Price per minute', 'quickcab') . ':'
        )
      );
    ?></tbody><?php

    do_action('quickcab_vehicle_metabox_after_vehicle_capacity_pricing_info', $post_id, $is_new, $p_struct);

    ?><tbody id="quickcab_hourly_pricing_options" <?php if (in_array($p_struct, $hide_hourly_rate_options) || $is_new) { ?>style="display:none;"<?php } ?>><?php

      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_hour',
        $post_id,
        array(
          'text' => esc_html__('Price per hour', 'quickcab') . ':'
        ),
        array(),
        apply_filters('quickcab_price_per_hour_helper_text', array(), $p_struct)
      );
    ?></tbody><?php

    do_action('quickcab_vehicle_metabox_after_price_per_hour', $post_id);

    ?><tbody><?php
      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_occupant',
        $post_id,
        array(
          'text' => esc_html__('Price per passenger', 'quickcab') . ':'
        )
      );

      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_suitcase',
        $post_id,
        array(
          'text' => esc_html__('Price per suitcase', 'quickcab') . ':'
        )
      );

      quickcab_render_vehicle_metabox_select_row(
        'quickcab_vehicle_waiting_unit',
        $post_id,
        array(
          'text' => esc_html__('Waiting Unit', 'quickcab') . ':'
        ),
        array(
          'options' => array(
            'hours' => esc_html__('Hours', 'quickcab'),
            'minutes' => esc_html__('Minutes', 'quickcab')
          )
        ),
        array(
          'text' => esc_html__('The unit used to price the waiting time on this vehicle.', 'quickcab')
        )
      );

      quickcab_render_vehicle_metabox_price_row(
        'quickcab_vehicle_price_per_waiting_unit',
        $post_id,
        array(
          'text' => sprintf(
            __('Price per waiting %s', 'quickcab') . ':',
            '<span class="quickcab-vehicle-extra-time-unit">' . ($is_new ? esc_html__('hour', 'quickcab') : $vehicle->get_waiting_unit_i18n()) . '</span>'
          )
        ),
        array(
          'suffix' => '/ <span class="quickcab-vehicle-extra-time-unit -margin">' . ($is_new ? esc_html__('hour', 'quickcab') : $vehicle->get_waiting_unit_i18n()) . '</span>'
        ),
        array(
          'text' => esc_html__('The price per extra hour of waiting the customer has selected', 'quickcab')
        )
      );
    ?></tbody><?php

    do_action('quickcab_vehicle_metabox_after_price_per_suitcase', $post_id);
  ?></table><?php
}
add_action('quickcab_meta_box_vehicle_price_fields', 'quickcab_render_vehicle_price_fields', 10);

function quickcab_render_vehicle_options_fields($post_id) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';
  $vehicle = new QuickCab_Vehicle($post_id);

  ?><table class="form-table">
    <tbody><?php
      quickcab_render_vehicle_metabox_number_row(
        'quickcab_vehicle_occupant_space',
        $post_id,
        array(
          'text' => esc_html__('Passenger Capacity', 'quickcab') . ':'
        ),
        array(
          'step'     => 1,
          'min'      => 1,
          'required' => true
        ),
        array(
          'text' => esc_html__('The maximum number of passengers you can fit in this vehicle', 'quickcab')
        )
      );

      quickcab_render_vehicle_metabox_number_row(
        'quickcab_vehicle_suitcase_space',
        $post_id,
        array(
          'text' => esc_html__('Suitcase Capacity', 'quickcab') . ':'
        ),
        array(
          'step'     => 1,
          'min'      => 0,
          'required' => true
        ),
        array(
          'text' => esc_html__('The maximum number of suitcases you can fit in this vehicle', 'quickcab')
        )
      );

      quickcab_render_vehicle_metabox_number_row(
        'quickcab_vehicle_number_owned',
        $post_id,
        array(
          'text' => esc_html__('Number Owned', 'quickcab') . ':'
        ),
        array(
          'step'     => 1,
          'min'      => 1,
          'required' => true
        ),
        array(
          'text' => esc_html__('The total number of these vehicles you own', 'quickcab')
        )
      );

      $calendar_color = '';

      if ( $is_new ) {
        $calendar_color = quickcab_random_color();
      } else if ( empty($vehicle->get_calendar_color()) ) {
        $calendar_color = '#1080f2';
      }

      quickcab_render_vehicle_metabox_colorpicker_row(
        'quickcab_vehicle_calendar_color',
        $post_id,
        array(
          'text' => esc_html__('Calendar Color', 'quickcab') . ':'
        ),
        array(
          'value' => $calendar_color
        ),
        array(
          'text' => esc_html__('The color bookings for this vehicle will appear in the calendar', 'quickcab')
        )
      );
    ?></tbody>
  </table><?php
}
add_action('quickcab_meta_box_vehicle_option_fields', 'quickcab_render_vehicle_options_fields', 10);

function quickcab_vehicle_metabox_fields() {
  return apply_filters('quickcab_metabox_vehicle_fields_save', array(
    'quickcab_vehicle_pricing_structure',
    'quickcab_vehicle_starting_price',
    'quickcab_vehicle_price_per_distance',
    'quickcab_vehicle_price_per_distance_from_base',
    'quickcab_vehicle_price_per_hour',
    'quickcab_vehicle_price_per_minute',
    'quickcab_vehicle_price_per_occupant',
    'quickcab_vehicle_price_per_suitcase',
    'quickcab_vehicle_occupant_space',
    'quickcab_vehicle_suitcase_space',
    'quickcab_vehicle_number_owned',
    'quickcab_vehicle_calendar_color',
    'quickcab_vehicle_waiting_unit',
    'quickcab_vehicle_price_per_waiting_unit'
  ));
}

function quickcab_vehicle_meta_box_save($post_id, $post) {

  if ( !isset($_POST['quickcab_vehicle_meta_box_nonce']) || !wp_verify_nonce(wp_unslash($_POST['quickcab_vehicle_meta_box_nonce']), basename(__FILE__)) ) {
    return;
  }

  if ( (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || (defined('DOING_AJAX') && DOING_AJAX) || isset($_REQUEST['bulk_edit']) ) {
    return;
  }

  if ( isset($post->post_type) && 'revision' == $post->post_type ) {
    return;
  }

  $vehicle = new QuickCab_Vehicle($post_id);

  // The default fields that get saved
  foreach ( quickcab_vehicle_metabox_fields() as $field ) {
    $meta_name = str_replace('quickcab_vehicle_', '', $field);

    if ( empty($_POST[$field]) ) {
      $vehicle->delete_meta($meta_name);
    } else {
      $new = apply_filters('quickcab_vehicle_metabox_save_' . $meta_name, wp_unslash($_POST[$field]), $post_id);
      $vehicle->update_meta($meta_name, $new);
    }
  }

  $occupant_space = $vehicle->get_occupant_space();
  $suitcase_space = $vehicle->get_suitcase_space();

  if (
    empty(get_option('quickcab_max_occupant_space')) ||
    $occupant_space > intval(get_option('quickcab_max_occupant_space'))
  ) {
    update_option('quickcab_max_occupant_space', $occupant_space);
  }

  if (
    empty(get_option('quickcab_max_suitcase_space')) ||
    $suitcase_space > intval(get_option('quickcab_max_suitcase_space'))
  ) {
    update_option('quickcab_max_suitcase_space', $suitcase_space);
  }

  do_action('quickcab_save_vehicle', $post_id, $post);
}
add_action('save_post', 'quickcab_vehicle_meta_box_save', 10, 2);

/**
 * When a QuickCab_Vehicles post status is changed (e.g. trashed or untrashed) then
 * the options for max suitcase and occupant capacity need to be recalculated
 *
 * @since   1.0
 * @param   string     $new_status  the new post status
 * @param   string     $old_status  the old post status
 * @param   WP_Post    $post        the WP_Post object
 * @return  void
 */
function quickcab_recalculate_vehicle_capacity_options($new_status, $old_status, $post) {
  if ( get_post_type($post) !== 'quickcab_vehicle' ) {
    return;
  }

  $vehicles = QuickCab_Vehicle::find_all();

  delete_option('quickcab_max_occupant_space');
  delete_option('quickcab_max_suitcase_space');

  foreach ( $vehicles as $vehicle ) {
    $occupant_space = $vehicle->get_occupant_space();
    $suitcase_space = $vehicle->get_suitcase_space();

    if (
      empty(get_option('quickcab_max_occupant_space')) ||
      $occupant_space > intval(get_option('quickcab_max_occupant_space'))
    ) {
      update_option('quickcab_max_occupant_space', $occupant_space);
    }

    if (
      empty(get_option('quickcab_max_suitcase_space')) ||
      $suitcase_space > intval(get_option('quickcab_max_suitcase_space'))
    ) {
      update_option('quickcab_max_suitcase_space', $suitcase_space);
    }
  }
}
add_action('transition_post_status', 'quickcab_recalculate_vehicle_capacity_options', 10, 3);
