<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Geofence_Shape {

  protected $id;
  protected $name;

  protected $db;

  public function __construct($value = false, $field = 'id') {
    $this->db = new QuickCab_Geofence_Shape_DB;

    if ( $value === false ) {
      return false;
    }

    $geofence_shape = $this->db->get_by($field, $value);

    if ( empty($geofence_shape) ) {
      return false;
    }

    $this->setup_geofence_shape($geofence_shape);
  }

  public function get_id() {
    return (int) $this->id;
  }

  public function get_name() {
    return apply_filters('quickcab_note_get_name', $this->name, $this->get_id());
  }

  public function get_coordinates() {
    $db = new QuickCab_Geofence_Shape_Coord_DB;
    return $db->get_geofence_shape_coords(array(
      'geofence_shape_id' => $this->get_id()
    ));
  }

  public function get_serialised_coordinates() {
    $coords = $this->get_coordinates();
    $formatted_coords = array();

    foreach ( $coords as $coord ) {
      array_push(
        $formatted_coords,
        $coord->latitude . ',' . $coord->longitude
      );
    }

    return implode('|', $formatted_coords);
  }

  public function create($data = array()) {
    do_action('quickcab_before_create_geofence_shape', $data);

    $id = false;

    if ( $geofence_shape_id = $this->db->add($data) ) {
      $geofence_shape = $this->db->get_by('id', $geofence_shape_id);

      $this->setup_geofence_shape($geofence_shape);

      $id = $geofence_shape_id;
    }

    do_action('quickcab_after_create_geofence_shape', $id, $data);

    return $id;
  }

  public static function find_all_available() {
    global $wpdb;

    $shapes = array();
    $db = new QuickCab_Geofence_Shape_DB;
    $db_shapes = $db->find_all();

    foreach ( $db_shapes as $shape ) {
      $new_shape = new QuickCab_Geofence_Shape($shape->id);

      if ( !empty($new_shape->get_name()) && !empty($new_shape->get_serialised_coordinates()) ) {
        array_push($shapes, $new_shape);
      }
    }

    usort($shapes, array('QuickCab_Geofence_Shape', 'sort_available_shapes'));

    return $shapes;
  }

  /**
   * Sort the available shapes in alphabetical order by their name
   *
   * @since  1.0
   * @param  QuickCab_Geofence_Shape $a
   * @param  QuickCab_Geofence_Shape $b
   * @return int
   */
  private static function sort_available_shapes($a, $b) {
    return strcmp($a->get_name(), $b->get_name());
  }

  /**
   * Assigns the variables of the class
   *
   * @since  1.0
   * @param  object  $geofence_shape  the object returned by the wpdb query
   * @return boolean                  did it succeed?
   */
  private function setup_geofence_shape($geofence_shape) {
    if ( !is_object($geofence_shape) ) {
      return false;
    }

    foreach ( $geofence_shape as $key => $value ) {
      $this->$key = $value;
    }

    if ( empty($this->get_id()) ) {
      return false;
    }

    return true;
  }
}
