<?php

if ( !defined('ABSPATH') ) exit;

/**
 * Renders the price row for a metabox
 *
 * @since  1.0
 * @param  string $name           name of the field, used for html 'name' attribute and postmeta
 * @param  int    $post_id        ID of the post
 * @param  array  $label_options  options for the label
 * @param  array  $input_options  options for the input
 * @param  array  $helper_options optons for the helper text
 */
function quickcab_render_metabox_price_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  if ( !isset($input_options['prefix']) ) {
    $input_options['prefix'] = '';
  }

  $input_options['prefix'] .= get_woocommerce_currency_symbol();
  $input_options['round'] = quickcab_get_option('decimal_points_shown');

  quickcab_render_metabox_number_row($name, $post_id, $label_options, $input_options, $helper_options);
}

function quickcab_render_metabox_code_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['id'] = $name;
  $input_options['name'] = $name;
  $input_options['type'] = isset($input_options['type']) ? $input_options['type'] : "text";

  if ( !isset($input_options['value']) || empty($input_options['value']) ) {
    $input_options['value'] = '';
  }

  $default_input_options = array(
    'size' => 'regular',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $value = $all_input_options['value'];
  unset($all_input_options['type']);
  unset($all_input_options['value']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  $all_input_options['class'] .= ' ' . $all_input_options['size'] . '-text';
  unset($all_input_options['size']);
  
  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo wp_kses_post($prefix);
      ?></span><?php

      ?><code <?php array_walk($all_input_options, function($val, $key) { echo ' ' . esc_html($key) . '="' . esc_attr($val) . '"'; }); ?>><?php
        echo esc_html($value);
      ?></code><?php

      ?><span class="quickcab-input-suffix"><?php
        echo wp_kses_post($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php
          echo esc_html($all_helper_options['text']);
        ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the number input row for a metabox
 *
 * @since  1.0
 * @param  string $name           name of the field, used for html 'name' attribute and postmeta
 * @param  int    $post_id        ID of the post
 * @param  array  $label_options  options for the label
 * @param  array  $input_options  options for the input
 * @param  array  $helper_options optons for the helper text
 */
function quickcab_render_metabox_number_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['id'] = $name;
  $input_options['name'] = $name;
  $input_options['type'] = "number";

  if ( !isset($input_options['value']) || empty($input_options['value']) ) {
    $input_options['value'] = 0;
  }

  if ( !$is_new && isset($input_options['round']) ) {
    $input_options['value'] = round($input_options['value'], $input_options['round']);
    unset($input_options['round']);
  }

  $default_input_options = array(
    'min'  => '0',
    'step' => '0.01',
    'size' => 'small',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  $all_input_options['class'] .= ' ' . $all_input_options['size'] . '-text';
  unset($all_input_options['size']);

  $disabled = false;

  if ( isset($all_input_options['disabled']) ) {
    $disabled = $all_input_options['disabled'];
    unset($all_input_options['disabled']);
  }

  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($name); ?>"><?php echo wp_kses_post($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo wp_kses_post($prefix);
      ?></span><?php

      ?><input <?php array_walk($all_input_options, function($val, $key) { echo ' ' . esc_html($key) . '="' . esc_attr($val) . '"'; }); ?> <?php if ($disabled) { ?>disabled<?php } ?>><?php

      ?><span class="quickcab-input-suffix"><?php
        echo wp_kses_post($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php
          echo esc_html($all_helper_options['text']);
        ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the string input row for a metabox
 *
 * @since  1.0
 * @param  string $name           name of the field, used for html 'name' attribute and postmeta
 * @param  int    $post_id        ID of the post
 * @param  array  $label_options  options for the label
 * @param  array  $input_options  options for the input
 * @param  array  $helper_options optons for the helper text
 */
function quickcab_render_metabox_string_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['id'] = $name;
  $input_options['name'] = $name;
  $input_options['type'] = isset($input_options['type']) ? $input_options['type'] : "text";

  if ( !isset($input_options['value']) || empty($input_options['value']) ) {
    $input_options['value'] = '';
  }

  $default_input_options = array(
    'size' => 'regular',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  if ( $all_input_options['type'] == 'textarea' ) {
    $value = $all_input_options['value'];
    unset($all_input_options['type']);
    unset($all_input_options['value']);
  }

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  $all_input_options['class'] .= ' ' . $all_input_options['size'] . '-text';
  unset($all_input_options['size']);

  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo esc_html($prefix);
      ?></span><?php

      if ($input_options['type'] == 'textarea') {
        ?><textarea <?php array_walk($all_input_options, function($val, $key) { echo ' ' . esc_html($key) . '="' . esc_attr($val) . '"'; }); ?>><?php
          echo esc_html($value);
        ?></textarea><?php
      } else {
        ?><input <?php array_walk($all_input_options, function($val, $key) { echo ' ' . esc_html($key) . '="' . esc_attr($val) . '"'; }); ?>><?php
      }

      ?><span class="quickcab-input-suffix"><?php
        echo esc_html($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php echo esc_html($all_helper_options['text']); ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the autocomplete row for a metabox
 *
 * @since  1.0
 * @param  string $name           [description]
 * @param  int    $post_id        [description]
 * @param  array  $label_options  [description]
 * @param  array  $input_options  [description]
 * @param  array  $helper_options [description]
 * @return void
 */
function quickcab_render_metabox_autocomplete_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['name'] = $name;

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = '';
  }

  $default_input_options = array(
    'size' => 'regular',
    'value' => '',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  $all_input_options['class'] .= ' ' . $all_input_options['size'] . '-text quickcab-google-autocomplete-field';
  unset($all_input_options['size']);

  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo esc_html($prefix);
      ?></span>

      <input type="text" class="<?php echo esc_attr($all_input_options['class']); ?>" value="<?php echo esc_attr($all_input_options['value']); ?>">
      <input type="hidden" value="<?php echo esc_attr($all_input_options['value']); ?>" id="<?php echo esc_attr($all_input_options['name']); ?>" name="<?php echo esc_attr($all_input_options['name']); ?>" class="quickcab-hidden-autocomplete-field">
      <span class="dashicons dashicons-yes quickcab-autocomplete-tick <?php if ($all_input_options['value']) { echo'-green'; } ?>"></span>

      <span class="quickcab-input-suffix"><?php
        echo esc_html($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php echo esc_html($all_helper_options['text']); ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the colorpicker row for a metabox
 *
 * @since  1.1.0
 * @param  string $name           [description]
 * @param  int    $post_id        [description]
 * @param  array  $label_options  [description]
 * @param  array  $input_options  [description]
 * @param  array  $helper_options [description]
 * @return void
 */
function quickcab_render_metabox_colorpicker_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['name'] = $name;

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = '';
  }

  $default_input_options = array(
    'value' => '',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);
  
  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($name); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo esc_html($prefix);
      ?></span>

      <input type="text" class="<?php echo sanitize_html_class($all_input_options['class']); ?> quickcab-color-picker" id="<?php echo esc_attr($all_input_options['name']); ?>" name="<?php echo esc_attr($all_input_options['name']); ?>" value="<?php echo esc_attr($all_input_options['value']); ?>" />

      <span class="quickcab-input-suffix"><?php
        echo esc_html($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php echo esc_html($all_helper_options['text']); ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the select row for a metabox
 *
 * @since  1.1.0
 * @param  string $name           [description]
 * @param  int    $post_id        [description]
 * @param  array  $label_options  [description]
 * @param  array  $input_options  [description]
 * @param  array  $helper_options [description]
 * @return void
 */
function quickcab_render_metabox_select_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['name'] = $name;

  if ( !isset($input_options['value']) ) {
    $input_options['value'] = '';
  }

  $default_input_options = array(
    'value' => '',
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($all_input_options['name']); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo esc_html($prefix);
      ?></span>

      <select id="<?php echo esc_attr($all_input_options['name']); ?>" name="<?php echo esc_attr($all_input_options['name']); ?>" <?php if (isset($all_input_options['disabled']) && $all_input_options['disabled']) { ?>disabled<?php } ?>><?php
        foreach ( $all_input_options['options'] as $key => $select_option ) {
          ?><option value="<?php echo esc_attr($key); ?>" <?php if ($all_input_options['value'] == $key) { ?>selected="selected"<?php } ?>><?php
            echo esc_html($select_option);
          ?></option><?php
        }
      ?></select>

      <span class="quickcab-input-suffix"><?php
        echo esc_html($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php echo esc_html($all_helper_options['text']); ?></p><?php
      }
    ?></td>
  </tr><?php
}

/**
 * Renders the checkbox row for a metabox
 *
 * @since  1.1.0
 * @param  string $name           [description]
 * @param  int    $post_id        [description]
 * @param  array  $label_options  [description]
 * @param  array  $input_options  [description]
 * @param  array  $helper_options [description]
 * @return void
 */
function quickcab_render_metabox_checkbox_row($name, $post_id, $label_options = array(), $input_options = array(), $helper_options = array()) {
  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  if ( !isset($input_options['class']) ) {
    $input_options['class'] = '';
  }
  $input_options['name'] = $name;

  if ( !isset($input_options['checked']) ) {
    $input_options['checked'] = false;
  }

  $default_input_options = array(
    'prefix' => '',
    'suffix' => ''
  );

  $all_input_options = array_merge($default_input_options, $input_options);

  $prefix = $all_input_options['prefix'];
  $suffix = $all_input_options['suffix'];

  unset($all_input_options['prefix']);
  unset($all_input_options['suffix']);

  $default_label_options = array(
    'text'  => ''
  );

  $all_label_options = array_merge($default_label_options, $label_options);

  $default_helper_options = array(
    'text' => '',
    'show_on_load' => true,
    'id' => ''
  );

  $all_helper_options = array_merge($default_helper_options, $helper_options);

  ?><tr>
    <th scope="row">
      <label for="<?php echo esc_attr($all_input_options['name']); ?>"><?php echo esc_html($all_label_options['text']); ?></label>
    </th>
    <td>
      <span class="quickcab-input-prefix"><?php
        echo esc_html($prefix);
      ?></span>

      <input type="checkbox" name="<?php echo esc_attr($all_input_options['name']); ?>" id="<?php echo esc_attr($all_input_options['name']); ?>" value="1" <?php if ($all_input_options['checked']) { ?>checked<?php } ?>>

      <span class="quickcab-input-suffix"><?php
        echo esc_html($suffix);
      ?></span><?php

      if ( !empty($all_helper_options['text']) ) {
        ?><p class="description" id="<?php echo esc_attr($all_helper_options['id']); ?>" <?php if (!$all_helper_options['show_on_load']) { ?>style="display:none;"<?php } ?>><?php echo esc_html($all_helper_options['text']); ?></p><?php
      }
    ?></td>
  </tr><?php
}
