<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_tiered_pricing_vehicle_pricing_options($array) {
  $units = quickcab_get_option('distance_units');

  return array_merge($array, array(
    'per-distance-tiered' => sprintf(
      esc_html__('Per %s (tiered)', 'quickcab'),
      quickcab_singular_units_word($units)
    ),
    'per-hour-tiered' => esc_html__('Per hour (tiered)', 'quickcab')
  ));
}
add_filter('quickcab_vehicle_pricing_options', 'quickcab_tiered_pricing_vehicle_pricing_options');

function quickcab_tiered_pricing_hide_starting_price_options($options) {
  return array_merge($options, array(
    'per-hour-tiered'
  ));
}
add_filter('quickcab_vehicle_pricing_options_hide_starting_price', 'quickcab_tiered_pricing_hide_starting_price_options');

function quickcab_tiered_pricing_hide_hourly_rate_options($options) {
  return array_merge($options, array(
    'per-distance-tiered'
  ));
}
add_filter('quickcab_vehicle_pricing_options_hide_hourly_rate', 'quickcab_tiered_pricing_hide_hourly_rate_options');

function quickcab_tiered_pricing_price_per_mile_helper_text($options, $p_struct) {
  $units = quickcab_get_option('distance_units');

  return array_merge($options, array(
    'text' => sprintf(
      esc_html__('This will be used as the price per %s if the distance exceeds the options you enter below.', 'quickcab'),
      quickcab_singular_units_word($units)
    ),
    'id' => 'quickcab_vehicle_price_per_distance_helper_text',
    'show_on_load' => $p_struct == 'per-distance-tiered'
  ));
}
add_filter('quickcab_price_per_mile_helper_text', 'quickcab_tiered_pricing_price_per_mile_helper_text', 10, 2);

function quickcab_tiered_pricing_price_per_hour_helper_text($options, $p_struct) {
  $units = quickcab_get_option('distance_units');

  return array_merge($options, array(
    'text' => esc_html__('This will be used as the price per hour if the journey time exceeds the options you enter below.', 'quickcab'),
    'id' => 'quickcab_vehicle_price_per_hour_helper_text',
    'show_on_load' => $p_struct == 'hourly-tiered'
  ));
}
add_filter('quickcab_price_per_hour_helper_text', 'quickcab_tiered_pricing_price_per_hour_helper_text', 10, 2);

function quickcab_tiered_pricing_tiered_distance_fields($post_id) {
  $units = quickcab_get_option('distance_units');
  $vehicle = new QuickCab_Vehicle($post_id);
  $p_struct = $vehicle->get_pricing_structure();

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  ?><tbody id="quickcab_modular_pricing_tiered_options" <?php if ($p_struct != 'per-distance-tiered') { ?>style="display:none;"<?php } ?>>
    <tr>
      <th scope="row">
        <label><?php
          printf(
            __('Price per %s:', 'quickcab'),
            quickcab_singular_units_word($units)
          );
        ?></label>
      </th>
      <td>
        <table class="wp-list-table fixed posts quickcab-tiered-table -distance">
          <thead>
            <tr>
              <th><?php
                printf(
                  __('Distance from (%s)', 'quickcab'),
                  $units
                );
              ?></th>
              <th><?php
                printf(
                  __('Distance to (%s)', 'quickcab'),
                  $units
                );
              ?></th>
              <th><?php
                echo apply_filters('quickcab_tiered_pricing_distance_price_header', sprintf(
                  __('Price per %s (%s)', 'quickcab'),
                  quickcab_singular_units_word($units),
                  get_woocommerce_currency_symbol()
                ));
              ?></th>
              <th></th>
            </tr>
          </thead>
          <tbody><?php
            $distance_tiers = $vehicle->get_price_per_distance_tiered();
            $before_currency = '';
            $after_currency = '';

            if ( $is_new || empty($distance_tiers) ) {
              ?><tr>
                <td>
                  <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_distance_tiered[0][value_from]" value="0" disabled>
                </td>
                <td>
                  <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_distance_tiered[0][value_to]" data-tiered-type="distance">
                </td>
                <td>
                  <?php echo get_woocommerce_currency_symbol(); ?><input type="number" min="0" step="0.01" class="small-text" name="quickcab_vehicle_price_per_distance_tiered[0][price]"><?php

                  do_action('quickcab_tiered_pricing_distance_after_price', null);
                ?></td>
                <td><?php
                  do_action('quickcab_tiered_pricing_distance_actions', null, 0);

                  ?><a href="#" class="button qc-pricing-tier-remove" data-tiered-type="distance"><?php
                    echo esc_html__('Remove Tier', 'quickcab');
                  ?></a>
                </td>
              </tr><?php
            } else {
              for ( $i = 0; $i < count($distance_tiers); $i++ ) {
                $distance_tier = $distance_tiers[$i];
                ?><tr>
                  <td>
                    <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_distance_tiered[<?php echo esc_attr($i); ?>][value_from]" value="<?php echo esc_attr($distance_tier['value_from']); ?>"  <?php if ($i == 0) { ?> value="0" disabled<?php } ?>>
                  </td>
                  <td>
                    <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_distance_tiered[<?php echo esc_attr($i); ?>][value_to]" value="<?php echo esc_attr($distance_tier['value_to']); ?>" data-tiered-type="distance">
                  </td>
                  <td>
                    <?php echo get_woocommerce_currency_symbol(); ?><input type="number" min="0" step="0.01" class="small-text" name="quickcab_vehicle_price_per_distance_tiered[<?php echo esc_attr($i); ?>][price]" value="<?php echo esc_attr($distance_tier['price']); ?>"><?php

                    do_action('quickcab_tiered_pricing_distance_after_price', $distance_tier);
                  ?></td>
                  <td><?php
                    do_action('quickcab_tiered_pricing_distance_actions', $distance_tier, $i);

                    ?><a href="#" class="button qc-pricing-tier-remove" data-tiered-type="distance"><?php
                      echo esc_html__('Remove Tier', 'quickcab');
                    ?></a>
                  </td>
                </tr><?php
              }
            }
          ?></tbody>
        </table>
        <a href="#" class="button" id="tiered_pricing_distance_add_tier"><?php
          echo esc_html__('Add Pricing Tier', 'quickcab');
        ?></a>
        <p class="description"><?php
          echo esc_html__('The value of the lower bounds must be the previous upper bound + 1 (if there is a previous tier)', 'quickcab');
        ?></p>
      </td>
    </tr>
  </tbody><?php
}
add_action('quickcab_vehicle_metabox_after_price_per_distance', 'quickcab_tiered_pricing_tiered_distance_fields', 10, 3);

function quickcab_tiered_pricing_tiered_hours_fields($post_id) {
  $vehicle = new QuickCab_Vehicle($post_id);
  $p_struct = $vehicle->get_pricing_structure();

  $screen = get_current_screen();
  $is_new = $screen->action == 'add';

  ?><tbody id="quickcab_hourly_pricing_tiered_options" <?php if ($p_struct != 'per-hour-tiered') { ?>style="display:none;"<?php } ?>>
    <tr>
      <th scope="row">
        <label><?php
          echo esc_html__('Pricing Tiers', 'quickcab') . ':';
        ?></label>
      </th>
      <td>
        <table class="wp-list-table fixed posts quickcab-tiered-table -hourly">
          <thead>
            <tr>
              <th><?php echo esc_html__('Hour from', 'quickcab'); ?></th>
              <th><?php echo esc_html__('Hour to', 'quickcab'); ?></th>
              <th><?php echo esc_html__('Price per Hour', 'quickcab'); ?></th>
              <th><?php echo esc_html__('Remove', 'quickcab'); ?></th>
            </tr>
          </thead>
          <tbody><?php
            $hourly_tiers = $vehicle->get_price_per_hour_tiered();
            $before_currency = '';
            $after_currency = '';

            if ( get_option('woocommerce_currency_pos') == 'left' || get_option('woocommerce_currency_pos') == 'left_space' ) {
              $before_currency = get_woocommerce_currency_symbol();
            } else {
              $after_currency = get_woocommerce_currency_symbol();
            }

            if ( $is_new || empty($hourly_tiers) ) {
              ?><tr>
                <td>
                  <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_hour_tiered[0][value_from]" value="0" disabled>
                </td>
                <td>
                  <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_hour_tiered[0][value_to]" data-tiered-type="distance">
                </td>
                <td>
                  <?php echo esc_html($before_currency); ?><input type="number" min="0" step="0.01" class="small-text" name="quickcab_vehicle_price_per_hour_tiered[0][price]"><?php echo esc_html($after_currency); ?>
                </td>
                <td>
                  <a href="#" class="button qc-pricing-tier-remove" data-tiered-type="hourly"><?php
                    echo esc_html__('Remove Tier', 'quickcab');
                  ?></a>
                </td>
              </tr><?php
            } else {
              for ( $i = 0; $i < count($hourly_tiers); $i++ ) {
                $hourly_tier = $hourly_tiers[$i];

                ?><tr>
                  <td>
                    <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-from" name="quickcab_vehicle_price_per_hour_tiered[<?php echo esc_attr($i); ?>][value_from]" value="<?php echo esc_attr($hourly_tier['value_from']); ?>" <?php if ($i == 0) { ?> value="0" disabled<?php } ?>>
                  </td>
                  <td>
                    <input type="number" min="0" step="0.1" class="small-text quickcab-tiered-pricing-value-to" name="quickcab_vehicle_price_per_hour_tiered[<?php echo esc_attr($i); ?>][value_to]" value="<?php echo esc_attr($hourly_tier['value_to']); ?>" data-tiered-type="distance">
                  </td>
                  <td>
                    <?php echo esc_html($before_currency); ?><input type="number" min="0" step="0.01" class="small-text" name="quickcab_vehicle_price_per_hour_tiered[<?php echo esc_attr($i); ?>][price]" value="<?php echo esc_attr($hourly_tier['price']); ?>"><?php echo esc_html($after_currency); ?>
                  </td>
                  <td>
                    <a href="#" class="button qc-pricing-tier-remove" data-tiered-type="hourly"><?php
                      echo esc_html__('Remove Tier', 'quickcab');
                    ?></a>
                  </td>
                </tr><?php
              }
            }
          ?></tbody>
        </table>
        <a href="#" class="button" id="tiered_pricing_hourly_add_tier"><?php
          echo esc_html__('Add Pricing Tier', 'quickcab');
        ?></a>
        <p class="description"><?php
          echo esc_html__('The value of the lower bounds must be the previous upper bound + 1 (if there is a previous tier)', 'quickcab');
        ?></p>
      </td>
    </tr>
  </tbody><?php
}
add_action('quickcab_vehicle_metabox_after_price_per_hour', 'quickcab_tiered_pricing_tiered_hours_fields', 10, 3);

/**
 * Check if the vehicle uses tiered pricing, if so charge the appropriate amount
 *
 * @since  1.0
 * @param  array      $price_info  the price info array to be added to
 * @param  QuickCab_Vehicle $vehicle     the vehicle being quoted for
 * @param  QuickCab_Journey $journey     the journey being quoted for
 * @return array                   the new price info array
 */
function quickcab_pro_check_tiered_pricing($price_info, $vehicle, $journey) {
  if ( $vehicle->get_pricing_structure() == 'per-distance-tiered' ) {
    $remaining_distance = $journey->get_distance_in_given_unit();
    $distance_covered = 0;

    foreach ( $vehicle->get_price_per_distance_tiered_for_price_calculation() as $distance_tier ) {
      $this_tier_distance = $distance_tier['value_to'] - $distance_tier['value_from'];
      $chargable_distance = min($remaining_distance, $this_tier_distance);

      $distance_covered += $chargable_distance;
      $remaining_distance -= $chargable_distance;

      $price_info = quikcab_add_to_price_info(
        $price_info,
        'distance',
        quickcab_get_option('price_breakdown_distance_label'),
        $chargable_distance * $distance_tier['price'],
        esc_html__('Tiered', 'quickcab')
      );

      if ( $remaining_distance <= 0 ) {
        break;
      }
    }

    // if distance tiers dont cover all distance, resort to default
    if ( $journey->get_distance_in_given_unit() > $distance_covered ) {
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'distance',
        quickcab_get_option('price_breakdown_distance_label'),
        ($journey->get_distance_in_given_unit() - $distance_covered) * $vehicle->get_price_per_distance(),
        esc_html__('Tiered', 'quickcab')
      );
    }

    $price_info = apply_filters('quickcab_price_after_distance_tiered_calculation', $price_info, $vehicle, $journey);
  } else if ( $vehicle->get_pricing_structure() == 'per-hour-tiered' ) {
    $remaining_hours = ceil($journey->get_duration_in_hours());
    $hours_covered = 0;

    foreach ( $vehicle->get_price_per_hour_tiered() as $hourly_tier ) {
      $this_tier_hours = $hourly_tier['value_to'] - $hourly_tier['value_from'];
      $chargable_hours = min($remaining_hours, $this_tier_hours);

      $hours_covered += $chargable_hours;
      $remaining_hours -= $chargable_hours;

      $price_info = quikcab_add_to_price_info(
        $price_info,
        'duration',
        quickcab_get_option('price_breakdown_duration_label'),
        $chargable_hours * $hourly_tier['price'],
        esc_html__('Tiered', 'quickcab')
      );

      if ( $remaining_hours <= 0 ) {
        break;
      }
    }

    // If hour tiers dont cover all hours, resort to default
    if ( $journey->get_duration_in_hours() > $hours_covered ) {
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'duration',
        quickcab_get_option('price_breakdown_duration_label'),
        ($journey->get_duration_in_hours() - $hours_covered) * $vehicle->get_price_per_hour(),
        esc_html__('Tiered', 'quickcab')
      );
    }

    $price_info = apply_filters('quickcab_price_after_per_hour_tiered_calculation', $price_info, $vehicle, $journey);
  }

  return $price_info;
}
add_filter('quickcab_price_after_all_vehicle_costs_calculation', 'quickcab_pro_check_tiered_pricing', 9, 3);

/**
 * Update all the pricing tiers for the given vehicle
 */
function quickcab_tiered_pricing_save_vehicle($post_id, $post) {
  global $wpdb;

  $tiered_pricing_table = $wpdb->prefix . 'quickcab_tiered_pricing';

  $wpdb->delete(
    $tiered_pricing_table,
    array(
      'vehicle_id' => $post_id
    )
  );

  $count = 0;

  foreach ( $_POST['quickcab_vehicle_price_per_distance_tiered'] as $distance_tier ) {

    if ( $count == 0 ) {
      $distance_tier['value_from'] = 0;
    }

    if ( !isset($distance_tier['value_from']) || !isset($distance_tier['value_to']) || !isset($distance_tier['price']) ) {
      continue;
    }

    $distance_tier_data = apply_filters('quickcab_distance_tier_save_data', array(
      'value_from' => esc_sql($distance_tier['value_from']),
      'value_to'   => esc_sql($distance_tier['value_to']),
      'price'      => esc_sql($distance_tier['price']),
      'type'       => 'distance',
      'vehicle_id' => $post_id
    ), $distance_tier);

    $insert_query = $wpdb->insert(
      $tiered_pricing_table,
      $distance_tier_data
    );

    $count++;
  }

  $count = 0;

  foreach ( $_POST['quickcab_vehicle_price_per_hour_tiered'] as $hourly_tier ) {

    if ( $count == 0 ) {
      $hourly_tier['value_from'] = 0;
    }

    if ( !isset($hourly_tier['value_from']) || !isset($hourly_tier['value_to']) || !isset($hourly_tier['price']) ) {
      continue;
    }

    $hourly_tier_tier_data = apply_filters('quickcab_hourly_tier_save_data', array(
      'value_from' => esc_sql($hourly_tier['value_from']),
      'value_to'   => esc_sql($hourly_tier['value_to']),
      'price'      => esc_sql($hourly_tier['price']),
      'type'       => 'hourly',
      'vehicle_id' => $post_id
    ), $hourly_tier);

    $insert_query = $wpdb->insert(
      $tiered_pricing_table,
      $hourly_tier_tier_data
    );

    $count++;
  }
}
add_action('quickcab_save_vehicle', 'quickcab_tiered_pricing_save_vehicle', 10, 2);

/**
 * Charge per occupant/suitcase/minute etc. for these pricing modes
 */
function quickcab_tiered_pricing_add_standard_pricing_mode($modes) {
  return array_merge($modes, array('per-distance-tiered'));
}
add_action('quickcab_standard_pricing_modes', 'quickcab_tiered_pricing_add_standard_pricing_mode');
