<?php

if ( !defined('ABSPATH') ) exit;

function quickcab_get_event_tooltip_title($quickcab_order) {

  ob_start();

  ?><div>
    <a href="<?php echo esc_attr(get_edit_post_link($quickcab_order->get_ID())); ?>"><?php
      echo esc_html__('Order', 'quickcab') . ' #' . $quickcab_order->get_ID();
    ?></strong>
  </div><?php

  $output_string = ob_get_contents();
  ob_end_clean();

  return $output_string;
}

function quickcab_get_event_tooltip_content($quickcab_order, $is_return = false) {
  $wc_order = $quickcab_order->get_wc_order();

  ob_start();

  ?><div>
    <strong><?php
      echo esc_html__('Collection Address', 'quickcab') . ': ';
    ?></strong><?php

    if ( $is_return ) {
      echo esc_html($quickcab_order->get_destination_address());
    } else {
      echo esc_html($quickcab_order->get_starting_address());
    }
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Destination Address', 'quickcab') . ': ';
    ?></strong><?php

    if ( $is_return ) {
      echo esc_html($quickcab_order->get_starting_address());
    } else {
      echo esc_html($quickcab_order->get_destination_address());
    }
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Vehicle', 'quickcab') . ': ';
    ?></strong>

    <a href="<?php echo esc_attr(get_edit_post_link($quickcab_order->get_vehicle_id())); ?>" target="_blank"><?php
      echo esc_html($quickcab_order->get_vehicle_name());
    ?></a>
  </div>

  <div>
    <strong><?php
      echo esc_html__('Collection Time', 'quickcab') . ': ';
    ?></strong><?php

    if ( $is_return ) {
      echo esc_html($quickcab_order->get_return_departure_time_in_format());
    } else {
      echo esc_html($quickcab_order->get_departure_time_in_format());
    }
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Arrival Time', 'quickcab') . ' (' . esc_html__('estimate', 'quickcab') . '): ';
    ?></strong><?php

    if ( $is_return ) {
      echo esc_html($quickcab_order->get_estimated_return_arrival_time_in_format());
    } else {
      echo esc_html($quickcab_order->get_estimated_arrival_time_in_format());
    }
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Journey Duration', 'quickcab') . ': ';
    ?></strong><?php

    if ( $is_return ) {
      if ( empty($quickcab_order->get_return_duration_in_words_traffic()) ) {
        echo esc_html($quickcab_order->get_return_duration_in_words());
      } else {
        echo esc_html($quickcab_order->get_return_duration_in_words_traffic());
      }
    } else {
      if ( empty($quickcab_order->get_duration_in_words_traffic()) ) {
        echo esc_html($quickcab_order->get_duration_in_words());
      } else {
        echo esc_html($quickcab_order->get_duration_in_words_traffic());
      }
    }
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Passengers', 'quickcab') . ': ';
    ?></strong><?php

    echo esc_html($quickcab_order->get_occupants());
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Suitcases', 'quickcab') . ': ';
    ?></strong><?php

    echo esc_html($quickcab_order->get_suitcases());
  ?></div>

  <div>
    <strong><?php
      echo esc_html__('Customer', 'quickcab') . ': ';
    ?></strong>

    <a href="<?php echo admin_url('admin.php?page=wc-admin&filter=single_customer&customers=' . $wc_order->get_user_id() . '&path=%2Fcustomers'); ?>"><?php
      echo esc_html($wc_order->get_billing_first_name() . ' ' . $wc_order->get_billing_last_name() . (empty($wc_order->get_billing_company()) ? '' : ' (' . $wc_order->get_billing_company() . ')'));
    ?></a>
  </div><?php

  $output_string = ob_get_contents();
  ob_end_clean();

  return $output_string;
}

function quickcab_get_calendar_events() {
  global $wpdb;
  global $woocommerce;

  $product_id = quickcab_get_option('wc_product_id');
  $events = array();

  $post_search_args = array(
    'post_type' => 'shop_order',
    'post_status' => array('wc-processing', 'wc-completed'),
    'posts_per_page' => -1
  );
  $posts = get_posts($post_search_args);

  foreach ( $posts as $customer_order ) {
    $order = new WC_Order($customer_order->ID);

    $unacceptable_status = array('wc-cancelled', 'wc-refunded', 'wc-failed');

    if (
      in_array($order->get_status(), $unacceptable_status) || // not acceptable e.g. cancelled
      get_post_status($order->get_id()) == 'trash' || // trashed
      !get_post_meta($order->get_id(), '_quickcab_order_starting_address', true) // not QuickCab order
    ) {
      continue;
    }

    $quickcab_order = new QuickCab_Order($order->get_id());
    $background_color = empty($quickcab_order->get_vehicle()->get_calendar_color()) ? '#1080f2' : $quickcab_order->get_vehicle()->get_calendar_color();
    $border_color = quickcab_adjust_hex_brightness($background_color, -50);
    $text_color = quickcab_get_text_color($background_color);

    $event = array(
      'id' => $order->get_id(),
      'title' => $quickcab_order->get_calendar_title(),
      'qtipTitle' => quickcab_get_event_tooltip_title($quickcab_order),
      'description' => quickcab_get_event_tooltip_content($quickcab_order),
      'start' => $quickcab_order->get_departure_time_in_format('Y-m-d\TH:i:s'),
      'end' => $quickcab_order->get_estimated_arrival_time_in_format('Y-m-d\TH:i:s'),
      'backgroundColor' => $background_color,
      'borderColor' => $border_color,
      'textColor' => $text_color,
      'url' => $order->get_edit_order_url()
    );

    array_push($events, $event);

    if ( $quickcab_order->is_return() ) {

      $return_event = array(
        'id' => $order->get_id(),
        'title' => $quickcab_order->get_calendar_return_journey_title(),
        'qtipTitle' => quickcab_get_event_tooltip_title($quickcab_order),
        'description' => quickcab_get_event_tooltip_content($quickcab_order, true),
        'start' => $quickcab_order->get_return_departure_time_in_format('Y-m-d\TH:i:s'),
        'end' => $quickcab_order->get_estimated_return_arrival_time_in_format('Y-m-d\TH:i:s'),
        'backgroundColor' => $background_color,
        'borderColor' => $border_color,
        'textColor' => $text_color,
        'url' => $order->get_edit_order_url()
      );

      array_push($events, $return_event);
    }
  }

  header('Content-type: application/json');

  echo json_encode($events);
  wp_die();
}
add_action('wp_ajax_quickcab_get_calendar_events', 'quickcab_get_calendar_events');
add_action('wp_ajax_nopriv_quickcab_get_calendar_events', 'quickcab_get_calendar_events');
